from __future__ import division
from __future__ import print_function
from __future__ import absolute_import
from builtins import str
from builtins import range
from past.utils import old_div

from pulp import LpVariable, LpBinary, lpSum, value, LpProblem, LpMaximize, LpAffineExpression

try:
    from src.dippy import DipSolStatOptimal, Solve
    from src.dippy.examples.gen_func import *
except ImportError:
    from coinor.dippy import DipSolStatOptimal, Solve
    from coinor.dippy.examples.gen_func import *

from .facility_location_func import *

args = parseArgs()

prob = formulate(args)

if debug_print_lp:
    prob.writeLP('facility_main.lp')
    for n, i in enumerate(LOCATIONS):
        prob.writeRelaxed(n, 'facility_relax%s.lp' % i);

if args.useCustomSolver:
    prob.relaxed_solver = solve_subproblem
if args.customInitRule == 'initOneEach':
    prob.init_vars = init_one_each
if args.customInitRule == 'initFirstFit':
    prob.init_vars = init_first_fit
if args.useCustomCuts:
    prob.generate_cuts = generate_weight_cuts
if args.rootHeuristic:
    prob.heuristics = heuristics
    prob.root_heuristic = True
if args.nodeHeuristic:
    prob.heuristics = heuristics
    prob.node_heuristic = True

dippyOpts = addDippyOpts(args)
    
dippyOpts['TolZero'] = '%s' % tol

Solve(prob, dippyOpts)

if prob.display_mode != 'off':
    numNodes = len(prob.Tree.get_node_list())
    if prob.Tree.attr['display'] == 'svg':
        prob.Tree.write_as_svg(filename = "facility_node%d" % (numNodes + 1), 
                               prevfile = "facility_node%d" % numNodes)
    prob.Tree.display()

# print solution
print("Optimal solution found!") 
print("************************************")
for i in prob.LOCATIONS:
    if prob.use_vars[i].varValue > 0:
        print("Location ", i, " is assigned: ", end=' ')
        print([j for j in prob.PRODUCTS if prob.assign_vars[(i, j)].varValue > 0])
print("************************************")
print()



'''
Created on Dec 29, 2013

@author: ted
'''
from __future__ import division
from builtins import str
from past.utils import old_div
from pulp import LpVariable, LpBinary, lpSum, value, LpProblem, LpMaximize
try:
    from src.dippy import Solve
    from src.dippy.examples.gen_func import *
except ImportError:
    from coinor.dippy import Solve
    from coinor.dippy.examples.gen_func import *

from .milp_func import *

args = parseArgs()

tol = pow(pow(2, -24), old_div(2.0, 3.0))

prob = formulate(args)

dippyOpts = addDippyOpts(args)

dippyOpts['TolZero'] = str(tol)
#dippyOpts['LogDebugLevel'] = '3'
#dippyOpts['LogLevel'] = '4'
#dippyOpts['LogDumpModel'] = '5'
#dippyOpts['ALPS'] = {'msgLevel' : 3}
    
Solve(prob, dippyOpts)

if prob.display_mode != 'off':
    prob.Tree.display()




from __future__ import print_function
from builtins import range
from pulp import *
try:
    from src.dippy import Solve
    from src.dippy.examples.gen_func import *
except ImportError:
    from coinor.dippy import Solve
    from coinor.dippy.examples.gen_func import *

from .cutting_stock_func import *
    
args = parseArgs()

prob = formulate(args.module)

if args.useCustomSolver:
    prob.relaxed_solver = solve_subproblem

dippyOpts = addDippyOpts(args)
    
Solve(prob, dippyOpts)

for i, var in list(prob.useVars.items()):
    if var.varValue:
        print("Use", i, var.varValue)

for pat in prob.PATTERNS:
    for i in prob.ITEMS:
        if prob.cutVars[(pat, i)].varValue:
            print("Pat", pat, "item", i, \
                  prob.cutVars[(pat, i)].varValue)

##for (pat, w), var in cutVars.items():
##    if var.varValue:
##        print "Pat", pat, "item", w, var.varValue





from __future__ import division
from __future__ import print_function
from __future__ import absolute_import
from builtins import str
from builtins import range
from past.utils import old_div
import importlib as ilib
import argparse

from pulp import LpVariable, LpBinary, lpSum, value, LpProblem, LpMaximize, LpAffineExpression

try:
    from src.dippy import DipProblem, DipSolStatOptimal
    from src.dippy.examples.gen_func import *
except ImportError:
    from coinor.dippy import DipProblem, DipSolStatOptimal
    from coinor.dippy.examples.gen_func import *

tol = pow(pow(2, -24), old_div(2.0, 3.0))

debug_print = False

debug_print_lp = False

def parseArgs():
    
    parser = argparse.ArgumentParser(
        description='Solve a facility location problem.')
    parser.add_argument('--module', '-m', metavar = 'module name', 
                        help='name of the Python module from which to import data')
    parser.add_argument('--useCustomSolver', action='store_true', 
                        help='enable custom subproblem solver')
    parser.add_argument('--customInitRule', choices=['initOneEach', 'initFirstFit'],
                        default = 'Default', 
                        help='enable custom method of initializing columns')
    parser.add_argument('--useCustomCuts', action='store_true', 
                        help='enable custom cut generation')
    parser.add_argument('--rootHeuristic', '-r', action='store_true', 
                        help='enable root heuristic')
    parser.add_argument('--nodeHeuristic', '-n', action='store_true', 
                        help='enable node heuristic')
    parser.add_argument('--numLocations', '-L', type=int, default = 5, metavar = 'L',
                        help = 'number of locations for instance specified on command line')
    parser.add_argument('--numProducts', '-P', type=int, default = 5, metavar = 'P',
                        help = 'number of products for instance specified on command line')
    parser.add_argument('--demands', '-D', nargs='*', default = [7, 5, 3, 2, 2],
                        help = 'list of demands for instances specified on command line')
    parser.add_argument('--fixedCosts', '-F', nargs='*', default = None,
                        help = 'list of fixed costs for instance specified on command line')
    parser.add_argument('--capacity', '-C', type=int, default = 8, metavar = 'C',
                        help = 'capacity of facilities for instance specified on command line')
    
    addDippyArgs(parser)

    args = parser.parse_args()

    if len(args.demands) != args.numProducts:
        raise ('Number of demands specified is not equal to number of products!')
    
    if args.fixedCosts != None and len(args.fixedCosts) != args.numLocations:
        raise ('Number of fixed costs specified is not equal to number of locations!')
    
    return(args)

def formulate(args):

    FIXED_COST = None
    ASSIGNMENTS = None
    ASSIGNMENT_COSTS = None

    if args.module:
        m = ilib.import_module(args.module)
        LOCATIONS = m.LOCATIONS
        PRODUCTS = m.PRODUCTS
        DEMAND = m.DEMAND
        CAPACITY = m.CAPACITY
        if hasattr(m, 'FIXED_COST'):
            FIXED_COST = m.FIXED_COST
        if hasattr(m, 'ASSIGNMENTS'):
            ASSIGNMENTS = m.ASSIGNMENTS
        if hasattr(m, 'ASSIGNMENT_COSTS'):
            ASSIGNMENT_COSTS = m.ASSIGNMENT_COSTS
    else:
        LOCATIONS = range(args.numLocations)
        PRODUCTS = range(args.numProducts)
        DEMAND = [int(i) for i in args.demands]
        CAPACITY = args.capacity
        if args.fixedCosts != None:
            FIXED_COST = [int(i) for i in args.fixedCosts]
        
    if FIXED_COST == None:
        FIXED_COST = {i:1 for i in LOCATIONS}
    if ASSIGNMENTS == None:
        ASSIGNMENTS = [(i, j) for i in LOCATIONS for j in PRODUCTS]
    if ASSIGNMENT_COSTS == None:
        ASSIGNMENT_COSTS = {i:0 for i in ASSIGNMENTS}

    prob = DipProblem("Facility Location")

    assign_vars = LpVariable.dicts("x", ASSIGNMENTS, 0, 1, LpBinary)
    use_vars    = LpVariable.dicts("y", LOCATIONS, 0, 1, LpBinary)

    prob += (lpSum(use_vars[i] * FIXED_COST[i] for i in LOCATIONS) +
             lpSum(assign_vars[j] * ASSIGNMENT_COSTS[j] for j in ASSIGNMENTS), 
             "min")

    # assignment constraints
    for j in PRODUCTS:
        prob += lpSum(assign_vars[(i, j)] for i in LOCATIONS) == 1

    # Aggregate capacity constraints
    for i in LOCATIONS:
        prob.relaxation[i] += lpSum(assign_vars[(i, j)] * DEMAND[j]
                                    for j in PRODUCTS) <= CAPACITY * use_vars[i]

    # Disaggregate capacity constraints
    for i, j in ASSIGNMENTS:
        prob.relaxation[i] += assign_vars[(i, j)] <= use_vars[i]

    prob.LOCATIONS = LOCATIONS
    prob.PRODUCTS = PRODUCTS
    prob.assign_vars = assign_vars
    prob.use_vars = use_vars

    return prob
        
def solve_subproblem(prob, key, redCosts, target):
    if debug_print:
        print("solve_subproblem...")
        print("reduced costs:")
        print(redCosts)
        print("target value:", target)
   
    loc = key

    # Calculate effective objective coefficient of products

    avars = [assign_vars[(loc, j)] for j in PRODUCTS]
    obj = [max(-redCosts[assign_vars[(loc, j)]], 0) for j in PRODUCTS]
    weights = [DEMAND[j] for j in PRODUCTS]
   
    # Use 0-1 KP to max. total effective value of products at location
    z, solution = knapsack01(obj, weights, CAPACITY)
   
    # Get the reduced cost of the knapsack solution
    if debug_print:
        print([(v, redCosts[v]) for v in avars])
        print(obj)
        print("z, solution =", z, solution)
        print("redCosts[use_vars[loc]] =", redCosts[use_vars[loc]])
        print("Fixed cost, rc", FIXED_COST[loc], redCosts[use_vars[loc]] - z)

    if redCosts[use_vars[loc]] > z + tol: # ... or an empty location is "useful"
        if debug_print:
            print("Zero solution is optimal")
        return DipSolStatOptimal, [{}]

    var_values = dict([(avars[i], 1) for i in solution])
    var_values[use_vars[loc]] = 1

    if debug_print:
        rcCheck = 0.0
        for v in list(var_values.keys()):
            rcCheck += redCosts[v] * var_values[v]
        print("Checking rc calc", redCosts[use_vars[loc]] - z, rcCheck) 
        print(var_values)

    return DipSolStatOptimal, [var_values]

def knapsack01(obj, weights, capacity):
    """ 0/1 knapsack solver, maximizes profit. weights and capacity integer """
        
    debug_subproblem = False
    
    assert len(obj) == len(weights)
    n = len(obj)
    if n == 0:
        return 0, []

    if debug_subproblem:
        relaxation = LpProblem('relaxation', LpMaximize)
        relax_vars = [str(i) for i in range(n)]
        var_dict   = LpVariable.dicts("", relax_vars, 0, 1, LpBinary)
        relaxation += (lpSum(var_dict[str(i)] * weights[i] for i in range(n)) 
                       <= capacity)
        relaxation += lpSum(var_dict[str(i)] * obj[i] for i in range(n))
        relaxation.solve()
        relax_obj = value(relaxation.objective)

        solution =  [i for i in range(n) if var_dict[str(i)].varValue > tol ]

        print(relax_obj, solution)


    c = [[0]*(capacity+1) for i in range(n)]
    added = [[False]*(capacity+1) for i in range(n)]
    # c [items, remaining capacity]
    # important: this code assumes strictly positive objective values
    for i in range(n):
        for j in range(capacity+1):
            if (weights[i] > j):
                c[i][j] = c[i-1][j]
            else:
                c_add = obj[i] + c[i-1][j-weights[i]]
                if c_add > c[i-1][j]:
                    c[i][j] = c_add
                    added[i][j] = True
                else:
                    c[i][j] = c[i-1][j]

    # backtrack to find solution
    i = n-1
    j = capacity

    solution = []
    while i >= 0 and j >= 0:
        if added[i][j]:
            solution.append(i)
            j -= weights[i]
        i -= 1
        
    return c[n-1][capacity], solution

def generate_weight_cuts(prob, sol):
    # Define mu and T for each knapsack
    mu = {}
    S = {}
    for i in LOCATIONS:
        mu[i] = CAPACITY
        S[i] = []
    
    # Use current assign_var values to assign items to locations
    assigning = True 
    while assigning:
        bestValue = 0
        bestAssign = None
        for i in LOCATIONS:
            for j in PRODUCTS:
                if j not in S[i]: # If this product is not in the subset
                    if (sol[assign_vars[(i, j)]] > bestValue) \
                    and (DEMAND[j] <= mu[i]):
                        # The assignment variable for this product is closer
                        # to 1 than any other product checked, and "fits" in
                        # this location's remaining space
                        bestValue = sol[assign_vars[(i, j)]]
                        bestAssign = (i, j)
        # Make the best assignment found across all products and locactions
        if bestAssign:
            (i,j) = bestAssign
            mu[i] -= DEMAND[j] # Decrease spare CAPACITY at this location
            S[i].append(j) # Assign this product to this location's set
        else:
            assigning = False # Didn't find anything to assign - stop

    # Generate the weight cuts from the sets found above
    new_cuts = []
    for i in LOCATIONS:
        if len(S[i]) > 0: # If an item assigned to this location
            con = LpAffineExpression() # Start a new constraint
            con += sum(DEMAND[j] * assign_vars[(i, j)] 
                            for j in S[i])
            con += sum(max(0, DEMAND[j] - mu[i]) *
                            assign_vars[(i, j)] for j in PRODUCTS 
                            if j not in S[i])
            new_cuts.append(con <= CAPACITY - mu[i])

    # Return the set of cuts we created to DIP
    if len(new_cuts) > 0:
        return new_cuts

def first_fit_heuristic():
    # Sort the items in descending weight order
    productReqs = [(DEMAND[j],j) for j in PRODUCTS]
    productReqs.sort(reverse=True)

    # Add items to locations, fitting in as much
    # as possible at each location.
    allLocations = []
    while len(productReqs) > 0:
        waste = CAPACITY
        currentLocation = []
        j = 0
        while j < len(productReqs):
            # Can we fit this product?
            if productReqs[j][0] <= waste:
                currentLocation.append(productReqs[j][1]) # index
                waste -= productReqs[j][0] # requirement
                productReqs.pop(j)
            else:
                # Try to fit next item
                j += 1
        allLocations.append((currentLocation, waste))

    # Return a list of tuples: ([products],waste)
    return allLocations

def first_fit():
    # Use generic first-fit heuristic that is shared
    # between heuristics and initial variable generation
    allLocations = first_fit_heuristic()

    # Convert to decision variable values
    sol = {}
    for i in LOCATIONS:
        for j in PRODUCTS:
            sol[assign_vars[(i, j)]] = 0
        sol[use_vars[i]] = 0

    index = 0
    for loc in allLocations:
        i = LOCATIONS[index]
        sol[use_vars[i]] = 1
        for j in loc[0]:
            sol[assign_vars[(i, j)]] = 1
        index += 1
        
    return sol

def frac_fit(xhat):
    # Initialise solution
    sol = {}
    waste = {}
    for i in LOCATIONS:
        for j in PRODUCTS: sol[assign_vars[(i, j)]] = 0
        sol[use_vars[i]] = 0
        waste[i] = 0
        
    # Get the list of non-zero fractional assignments
    fracAssigns = [ (xhat[assign_vars[(i, j)]], (i, j))
                     for i in LOCATIONS for j in PRODUCTS
                     if xhat[assign_vars[(i, j)]] > tol ]
    fracAssigns.sort()

    # Track which products and locations have been used
    notAllocated = dict((j,True) for j in PRODUCTS)
    notUsed      = dict((i,True) for i in LOCATIONS)
    while len(fracAssigns) > 0:
        fracX = fracAssigns.pop() # Get best frac. assignment left
        (i,j) = fracX[1]
        if notAllocated[j]:
            if notUsed[i]: # Create a new location if needed
                notUsed[i] = False
                sol[use_vars[i]] = 1
                waste[i] = CAPACITY
            if DEMAND[j] <= waste[i]: # Space left?
                sol[assign_vars[(i, j)]] = 1
                notAllocated[j] = False
                waste[i] -= DEMAND[j]
    
    # Allocate the remaining products
    unallocated = [(DEMAND[j],j) for j in PRODUCTS
                                      if notAllocated[j]]
    unallocated.sort(reverse=True)
    unused = [i for i in LOCATIONS if notUsed[i]]
    while len(unallocated) > 0:
        waste = CAPACITY
        index = 0
        loc = unused.pop()
        while index < len(unallocated):
            (j_req, j) = unallocated[index]
            if j_req <= waste:
                unallocated.pop(index)
                sol[assign_vars[(loc, j)]] = 1
                waste -= j_req
            else: index += 1
        sol[use_vars[loc]] = 1

    return sol

def heuristics(prob, xhat, cost):
    sols = []
    if prob.root_heuristic:
        prob.root_heuristic = False # Don't run twice
        sol = first_fit()
        sols.append(sol)
    if prob.node_heuristic:
        sol = frac_fit(xhat)
        sols.append(sol)

    if len(sols) > 0:
        return sols
      
def init_first_fit(prob):

    locations = first_fit_heuristic()
    bvs = []
    index = 0
    for loc in locations:
        i = LOCATIONS[index]
        if debug_print:
            print([(assign_vars[(i, j)], 1) for j in loc[0]])
        var_values = dict([(assign_vars[(i, j)], 1) for j in loc[0]])
        var_values[use_vars[i]] = 1
        dv = (loc[1], var_values)
        bvs.append((i, dv))
        index += 1
    if debug_print:
        print(bvs)
    return bvs

def init_one_each(prob):
    bvs = []
    if debug_print:
        print("LOCATIONS =", LOCATIONS)
    for index, loc in enumerate(LOCATIONS):
        lc = [PRODUCTS[index]]
        waste = CAPACITY - DEMAND[PRODUCTS[index]]
        var_values = dict([(assign_vars[(loc, j)], 1) for j in lc])
        var_values[use_vars[loc]] = 1

        dv = (waste, var_values)
        bvs.append((loc, dv))
    if debug_print:
        print(bvs)
    return bvs




from __future__ import division
from __future__ import print_function
from builtins import range
from past.utils import old_div
from pulp import LpVariable, LpBinary, lpSum, value, LpProblem, LpMaximize, LpAffineExpression

try:
    import src.dippy as dippy
    from src.dippy import DipSolStatOptimal
    import src.dippy.examples.cflp.facility_ex1 as data
except ImportError:
    import coinor.dippy as dippy
    from coinor.dippy import DipSolStatOptimal
    import coinor.dippy.examples.cflp.facility_ex1 as data

from math import floor, ceil

tol = pow(pow(2, -24), old_div(2.0, 3.0))
debug_print = False

REQUIREMENT = data.REQUIREMENT
PRODUCTS = data.PRODUCTS
LOCATIONS = data.LOCATIONS
CAPACITY = data.CAPACITY

print(REQUIREMENT)

prob = dippy.DipProblem("Bin Packing")

assign_vars = LpVariable.dicts("AtLocation",
              [(i, j) for i in data.LOCATIONS
                      for j in data.PRODUCTS],
              0, 1, LpBinary)
use_vars    = LpVariable.dicts("UseLocation",
              data.LOCATIONS, 0, 1, LpBinary)
waste_vars  = LpVariable.dicts("Waste",
              data.LOCATIONS, 0, data.CAPACITY)

# objective: minimise waste
prob += lpSum(waste_vars[i] for i in LOCATIONS), "min"

# assignment constraints
for j in PRODUCTS:
    prob += lpSum(assign_vars[(i, j)] for i in LOCATIONS) == 1

# Aggregate capacity constraints
for i in LOCATIONS:
    prob.relaxation[i] += lpSum(assign_vars[(i, j)] * REQUIREMENT[j]
                                for j in PRODUCTS) + waste_vars[i] \
                                    == CAPACITY * use_vars[i]

# Disaggregate capacity constraints
for i in LOCATIONS:
    for j in PRODUCTS:
        prob.relaxation[i] += assign_vars[(i, j)] <= use_vars[i]
        
# Ordering constraints
for index, location in enumerate(LOCATIONS):
    if index > 0:
        prob += use_vars[LOCATIONS[index-1]] >= use_vars[location]
        
# Anti-symmetry branches
def choose_antisymmetry_branch(prob, sol):
    num_locations = sum(sol[use_vars[i]] for i in LOCATIONS)
    up   = ceil(num_locations)  # Round up to next nearest integer 
    down = floor(num_locations) # Round down
    if  (up - num_locations   > tol) \
    and (num_locations - down > tol): # Is fractional?
        # Down branch: provide upper bounds, lower bounds are default
        down_branch_ub = dict([(use_vars[LOCATIONS[n]], 0) for
                                n in range(int(down), len(LOCATIONS))])
        # Up branch: provide lower bounds, upper bounds are default
        up_branch_lb = dict([(use_vars[LOCATIONS[n]], 1) for
                                n in range(0, int(up))])
        # Return the advanced branch to DIP
        return ({}, down_branch_ub, up_branch_lb, {})

#prob.branch_method = choose_antisymmetry_branch

def solve_subproblem(prob, index, redCosts, target):
    loc = index

    # Calculate effective objective coefficient of products
    effs = {}
    for j in PRODUCTS:
        effs[j] = (redCosts[assign_vars[(loc, j)]]
                   - redCosts[waste_vars[loc]] * REQUIREMENT[j])

    obj = [-effs[j] for j in PRODUCTS]
    weights = [REQUIREMENT[j] for j in PRODUCTS]
    
    # Use 0-1 KP to max. total effective value of products at location
    z, solution = knapsack01(obj, weights, CAPACITY)
    
    # Get the reduced cost of the knapsack solution and waste
    if debug_print:
        rc = (redCosts[use_vars[loc]] -z + 
             redCosts[waste_vars[loc]] * CAPACITY)
        waste = CAPACITY - sum(weights[i] for i in solution)
        rc += redCosts[waste_vars[loc]] * waste

    if redCosts[use_vars[loc]] > z + tol: # ... or an empty location is "useful"
        if debug_print:
            print("Zero solution is optimal")
        return DipSolStatOptimal, [{}]

    # Return the solution if the reduced cost is low enough
    var_values = [(assign_vars[(loc, i)], 1) for i in solution]
    var_values.append((use_vars[loc], 1))
    var_values.append((waste_vars[loc], waste))
            
    return DipSolStatOptimal, [var_values]

#prob.relaxed_solver = solve_subproblem

def knapsack01(obj, weights, capacity):
    """ 0/1 knapsack solver, maximizes profit. weights and capacity integer """
    assert len(obj) == len(weights)
    n = len(obj)

    if n == 0:
        return 0, []

    c = [[0]*(capacity+1) for i in range(n)]
    added = [[False]*(capacity+1) for i in range(n)]
    # c [items, remaining capacity]
    # important: this code assumes strictly positive objective values
    for i in range(n):
        for j in range(capacity+1):
            if (weights[i] > j):
                c[i][j] = c[i-1][j]
            else:
                c_add = obj[i] + c[i-1][j-weights[i]]
                if c_add > c[i-1][j]:
                    c[i][j] = c_add
                    added[i][j] = True
                else:
                    c[i][j] = c[i-1][j]

    # backtrack to find solution
    i = n-1
    j = capacity

    solution = []
    while i >= 0 and j >= 0:
        if added[i][j]:
            solution.append(i)
            j -= weights[i]
        i -= 1

    return c[n-1][capacity], solution

def first_fit_heuristic():
    # Sort the items in descending weight order
    productReqs = [(REQUIREMENT[j],j) for j in PRODUCTS]
    productReqs.sort(reverse=True)

    # Add items to locations, fitting in as much
    # as possible at each location.
    allLocations = []
    while len(productReqs) > 0:
        waste = CAPACITY
        currentLocation = []
        j = 0
        while j < len(productReqs):
            # Can we fit this product?
            if productReqs[j][0] <= waste:
                currentLocation.append(productReqs[j][1]) # index
                waste -= productReqs[j][0] # requirement
                productReqs.pop(j)
            else:
                # Try to fit next item
                j += 1
        allLocations.append((currentLocation, waste))
    # Return a list of tuples: ([products],waste)
    return allLocations

def first_fit(prob):
    locations = first_fit_heuristic()
    bvs = []
    index = 0
    for loc in locations:
        i = LOCATIONS[index]
        var_values = [(assign_vars[(i, j)], 1) for j in loc[0]]
        var_values.append((use_vars[i], 1))
        var_values.append((waste_vars[i], loc[1]))
        dv = dippy.DecompVar(var_values, None, loc[1])
        bvs.append((i, dv))
        index += 1
    return bvs

def one_each(prob):
    bvs = []
    for index, loc in enumerate(LOCATIONS):
        lc = [PRODUCTS[index]]
        waste = CAPACITY - REQUIREMENT[PRODUCTS[index]]
        var_values = [(assign_vars[(loc, j)], 1) for j in lc]
        var_values.append((use_vars[loc], 1))
        var_values.append((waste_vars[loc], waste))
        
        dv = dippy.DecompVar(var_values, None, waste)
        bvs.append((loc, dv))
    return bvs

#prob.init_vars = first_fit
##prob.init_vars = one_each

dippy.Solve(prob, {
    'TolZero': '%s' % tol,
    'doPriceCut': '1',
#    'generateInitVars': '1', 
})

# print solution
for i in LOCATIONS:
    if use_vars[i].varValue > tol:
        print("Location ", i, \
              " produces ", \
              [j for j in PRODUCTS
               if assign_vars[(i, j)].varValue > tol])



#!/usr/bin/env python


from __future__ import division
from future import standard_library
standard_library.install_aliases()
from builtins import str
from builtins import range
from past.utils import old_div
__title__   = 'Display graphic solution for the CVPMP and CVPCP'
__version__ = '1.0 Nov 2013'
__author__  = 'Dago Quevedo'
__email__   = 'dago@yalma.fime.uanl.mx'


import  string
from    tkinter import  * 

def draw(path, mytype, id):
    f = open(path,"r")
    V = []
    P = []
    
    for line in f:
        Z = line.split()
        V.append([int(Z[0]),int(Z[1]),int(Z[2]),float(Z[3]),float(Z[4])])
        if int(Z[0]) ==  1:
            P.append([int(Z[0]),int(Z[1]),int(Z[2]),float(Z[3]),float(Z[4])])
    
    f.close()

    x_max = max(V,key = lambda x:x[3])[3]
    y_max = max(V,key = lambda x:x[4])[4]
    x_min = min(V,key = lambda x:x[3])[3]
    y_min = min(V,key = lambda x:x[4])[4]
    
    #Normalizacion de valores

    if mytype == 1:
        scale = 600
        delta = 20

    if mytype == 0:
        scale = 650
        delta = 20
    

    for i in range(len(V)):
        V[i][3] = ((old_div((V[i][3] - x_min), (x_max - x_min))) * scale) + delta
        V[i][4] = ((old_div((V[i][4] - y_min), (y_max - y_min))) * scale) + delta
        if i  <  len(P):
            P[i][3] = ((old_div((P[i][3] - x_min), (x_max - x_min))) * scale) + delta
            P[i][4] = ((old_div((P[i][4] - y_min), (y_max - y_min))) * scale) + delta

    _d_ = 0
    
    C = ["#87CEFA","#C0C0C0","#FFA500","#DDA0DD","#9ACD32", "#9ACD32",
         "#E99699","#B8F7B8","#D8D8F9","#FAD6A5","#F4FBA6", "#FF434F",
         "#87CEFA","#C0C0C0","#FFA500","#DDA0DD","#9ACD32", "#9ACD32",
         "#E99699","#B8F7B8","#D8D8F9","#FAD6A5","#F4FBA6", "#FF434F",
         "#87CEFA","#C0C0C0","#FFA500","#DDA0DD","#9ACD32", "#9ACD32",
         "#E99699","#B8F7B8","#D8D8F9","#FAD6A5","#F4FBA6", "#FF434F",
         "#87CEFA","#C0C0C0","#FFA500","#DDA0DD","#9ACD32", "#9ACD32",
         "#E99699","#B8F7B8","#D8D8F9","#FAD6A5","#F4FBA6", "#FF434F",
         "#87CEFA","#C0C0C0","#FFA500","#DDA0DD","#9ACD32", "#9ACD32",
         "#E99699","#B8F7B8","#D8D8F9","#FAD6A5","#F4FBA6", "#FF434F",
         "#87CEFA","#C0C0C0","#FFA500","#DDA0DD","#9ACD32", "#9ACD32",
         "#E99699","#B8F7B8","#D8D8F9","#FAD6A5","#F4FBA6", "#FF434F"]
    
    root = Tk()
    root.wm_attributes("-topmost", 1)

    root.title("Solution " + str(type) + " - " + str(id))
    canvas = Canvas(root,width = scale + (delta * 2), height = scale + (delta * 2), 
                    bg = 'white')  
    canvas.pack(expand = YES, fill = BOTH)

    for _k_ in range(len(P)):
        k = P[_k_]
        for j in V:
            if j[2] == k[1]:
                if j[0] == 0:
                    canvas.create_line(
                        k[3] + 10,k[4] + 10, 
                        j[3] + 7, j[4] + 7, 
                        dash = (3,3))
                else:
                    canvas.create_line(
                        k[3] + 10,k[4] + 10, 
                        j[3] + 7,j[4] + 7,fill = "red",width = 1.5, 
                        dash = (3,3))
                

                if j[1] > 9:_d_ = 2
                else     :_d_ = 0
                canvas.create_oval(
                        j[3],j[4], 
                        j[3] + 15,j[4] + 15, 
                                   width = 1, fill = C[_k_])
                
                canvas.create_text(
                        j[3] + (5 - _d_),
                        j[4] + 7, 
                                   text = str(j[1]),anchor = "w",fill = "black", 
                                   font = ("Arial", 10))
        canvas.create_rectangle(
                k[3],k[4], 
                k[3] + 20,k[4] + 20, 
                width = 2, fill = C[_k_]) 

        if k[1] > 9:_d_ = 3
        else       :_d_ = 0
        canvas.create_text(
                 k[3] + (6 - _d_),k[4] + 10, 
                 text = str(k[1]),anchor = "w",fill = "black", 
                 font = ("Arial", 14))

    root.mainloop()




from __future__ import division
from __future__ import print_function
from __future__ import absolute_import
from past.utils import old_div
from .coke_func import *
import importlib as ilib

args = parseArgs()

m = ilib.import_module(args.module)
    
mine_trans = read_table(m.mine_trans_data, int)

cust_trans = read_table(m.cust_trans_data, int, transpose=True)

transport_costs = dict(mine_trans)
transport_costs.update(cust_trans)

cp = CokeProb(supply = m.mine_supply, demand = m.customer_demand,
              LOCATIONS = m.LOCATIONS, build_costs = m.build_costs,
              conversion_factor = m.convert,
              transport_costs = transport_costs)

prob = formulate(cp)

# Set a zero tolerance (Mike Saunders' "magic number")  
prob.tol = pow(pow(2, -24), old_div(2.0, 3.0))

xopt = solve(prob, args)

for l in cp.LOCATIONS:
    for s in cp.SIZES:
        if xopt[prob.buildVars[(l,s)]] > 0:
            print("Build %s %s (%s)" % \
                  (l, s, xopt[prob.buildVars[(l,s)]]))
print()

print_var_table(cp.MINES, cp.LOCATIONS, prob.flowVars)
print()
print_var_table(cp.LOCATIONS, cp.CUSTOMERS, prob.flowVars)

if prob.display_mode != 'off':
    numNodes = len(prob.Tree.get_node_list())
    if ((prob.Tree.attr['display'] == 'pygame') or 
        (prob.Tree.attr['display'] == 'xdot')):
        prob.Tree.display()



"""
How to use this file:

from _dippy import *

_Solve = Solve
def Solve(prob, params=None):
  # Set up prob here, must have DipAPI as a base class

  # call the Solve method from _dippy
  try:
    solList, dualList = _Solve(prob, processed)
    # solList  is a list of (col_name, value) pairs
    # dualList is a list of (row_name, value) pairs
  except:
    print "Error returned from _dippy"
    raise
"""
from builtins import object

class DipAPIError(Exception):
  """
  DipAPI Exception
  """
  pass

class DipAPI(object):

  def getObjective(self):
    """
    Return objective as a dictionary with variables as keys
    and (non-zero) coefficients as values
    """
    raise DipAPIError("Bad function definition, DipAPI.getObjective must be overwritten")

  def getRows(self, problem=None):
    """
    Return constraints as a list of dictionaries with variables as keys
    and (non-zero) coefficients as values. Constraints also have
    getName, getLb and getUb methods

    problem = None implies the master problem, otherwise problem
    is a subproblem
    """
    raise DipAPIError("Bad function definition, DipAPI.getRows must be overwritten")

  def getCols(self, problem=None):
    """
    Returns a list of variables. Variables have getName, getLb,
    getUb and isInteger methods

    problem = None implies the master problem, otherwise problem
    is a subproblem
    """
    raise DipAPIError("Bad function definition, DipAPI.getCols must be overwritten")

  def getMasterAsTuple(self):
    """
    Returns all the master problem data as a tuple of other
    "data gathering" functions
    """
    return (self.getObjective(),
            self.getRows(),
            self.getCols())

  def getRelaxAsTuple(self, problem):
    """
    Returns all the subproblem constraints and variables
    """
    return (self.getRows(problem),
            self.getCols(problem))

  def getRelaxsAsDict(self):
    """
    Returns the relaxation subproblems as a dictionary with keys as
    defined by the user and values as subproblems
    """
    raise DipAPIError("Bad function definition, DipAPI.getRelaxsAsDict must be overwritten")

  def chooseBranchSet(self, xhat):
    """
    Finds the best branch for a fractional solution

    Inputs:
    xhat (list of (variable, value) tuples) = list of solution values for all variables

    Output:
    down_lb, down_ub, up_lb, up_ub (tuple of (variable, value) dictionaries) =
    lower and upper bounds for down branch, lower and upper bounds for up branch
    """
    raise DipAPIError("Bad function definition, DipAPI.chooseBranchSet must be overwritten")

  def postProcessNode(self, output):
    """
    Returns information from the node that has just been processed.

    Inputs:
    output (list of (parameter, value) tuples) = list of output values from the node
    """
    raise DipAPIError("Bad function definition, DipAPI.postProcess must be overwritten")

  def solveRelaxed(self, key, redCostX, target):
    """
    Returns solutions to the whichBlock relaxed subproblem

    Inputs:  
    key (Python Object) = key of relaxed subproblem to be solved
    redCostX (list of (variable, value) tuples) = list of reduced costs for all variables
    target (float) = any total reduced cost less than the target is "good" (results in a negative cost column)

    Output:
	  status (integer) = status of the relaxation solve, will be one of
	    DipSolStatOptimal    = no better columns can be found
	    DipSolStatFeasible   = better columns can be found, but just use these for now
	    DipSolStatNoSolution = use any columns returned, but also use DIP's default column finder
    varList (list of (cost, reduced cost, list of (variable, value) dictionaries)) =
    solution for this relaxed subproblem expressed as a cost, reduced cost and
    dictionary of non-zero values for variables
    """
    raise DipAPIError("Bad function definition, DipAPI.solveRelaxed must be overwritten")

  def isUserFeasible(self, sol, tol):
    """
    Lets the user decide if an integer solution is really feasible

    Inputs:
    sol (list of (variable, value) tuples) = list of solution values for all variables
    tol (double) = zero tolerance

    Outputs:
    (boolean) = false if not feasible (generate cuts) or true if feasible
    """
    raise DipAPIError("Bad function definition, DipAPI.isUserFeasible must be overwritten")

  def generateCuts(self, node):
    """
    Lets the user generate cuts to remove fractional "pieces" of the node solution

    Inputs:
    node (list of (string, object) tuples) = list of node properties

    Output:
    cutList (list of LpConstraints) =
    cuts for this fractional solution expressed as a list LpConstraints,
    i.e., a dictionary with LpVariables as keys and (non-zero) coefficients
    as values with getName, getLb and getUb bound methods
    """
    raise DipAPIError("Bad function definition, DipAPI.generateCuts must be overwritten")

  def solveHeuristics(self, xhat, costX):
    """
    Lets the user generate (heuristic) solutions from a fractional solution

    Inputs:
    xhat  (list of (variable, value) tuples) = list of solution values for all variables
    costX (list of (variable, value) tuples) = list of costs for all variables

    Outputs:
    solList (list of (variable, value) dictionaries) =
    solutions found from this fractional solution expressed as a
    dictionary of non-zero values for variables
    """
    raise DipAPIError("Bad function definition, DipAPI.solveHeuristics must be overwritten")

  def generateInitVars(self):
    """
    Returns initial solutions to relaxed subproblems

    Inputs:
    None

    Output:
    varList (list of (subproblem key, (cost, list of (variable, value) dictionaries))) =
    initial solutions for the relaxed subproblems expressed as a cost and
    dictionary of non-zero values for variables
    """
    raise DipAPIError("Bad function definition, DipAPI.generateInitVars must be overwritten")



from os.path import dirname
from inspect import getfile
import coinor.dippy.examples

print('''
Welcome the DiPPy examples! Here you will find a wide range of examples.
The examples are installed in the directory
''')
print(dirname(getfile(coinor.dippy.examples)))
print(
'''
They're meant to be inspected, so please do look at the source code!
It's also available in the source repo

https://github.com/coin-or/Dip/tree/master/Dip/src/dippy/examples

Each example can be run directly as a Python module with a default input data, e.g.,

python -m coinor.dippy.examples.cflp

For help on how to run a given example, run the module with '--help'.
Most examples also accept input data in the form of a Python module, e.g.,

python -m coinor.dippy.examples.cflp coinor.dippy.examples.bpp.bpp_data

The example modules available are

bpp:     Bin Packing Problem
cflp:    Capacitated Facility Location Problem
coke:    Steel Manufacturing
csp:     Cutting Stock Problem
cvpmp:   Capacitated P-median
gap:     Generalized Assignment
milp:    Random Block Structured MILP
tsp:     Traveling Salesman Problem
wedding: Wedding Planner

Enjoy!
''')



from __future__ import absolute_import
from builtins import range
#!/usr/bin/env python


__title__   = 'Read instances for the CVPMP and CVPCP'
__version__ = '1.0 Nov 2013'
__author__  = 'Dago Quevedo'
__email__   = 'dago@yalma.fime.uanl.mx'


import string
from . import Global


def read(path):
    file = open(path,'r')
    line = file.readline().split()

    type = int(line[0])
    cxy  = None
    
    #1 - Beasly
    if type == 1:
        id   = int(line[1])
        n    = int(line[2])
        p    = int(line[3])
        line = file.readline().split()
        q    = int(line[0])
        
        cxy  = {}
        d    = {}
        s    = {}
        w    = {}
        V    = [i for i in range(1, n + 1)]
        
        for i in V:
            line    = file.readline().split()
            cxy[i]  = [int(line[1]), int(line[2])]
            s[i]    = q
            w[i]    = int(line[3])
        
        for i in range(1,n+1):
            for j in range(i,n+1):
                e = Global.euclidean(cxy[i][0], cxy[j][0], cxy[i][1], cxy[j][1])
                d[i,j] = e
                d[j,i] = e
    
    
    #2 - GalvaoReVelle
    if type == 2:
        id = int(line[1])
        n  = int(line[2])
        p  = int(line[3])
        
        d  = {}
        s  = {}
        w  = {}
        V  = [i for i in range(1,n+1)]
        
        i = 1
        for s_ in file.readline().split():
            s[i] = int(float(s_))
            i += 1
        
        i = 1
        for w_ in file.readline().split():
            w[i] = int(float(w_))
            i += 1
        
        for i in V:
            j = 1
            for d_ in file.readline().split():
                d[i,j] = int(float(d_))
                j += 1
    
    
    #3 - Lorena
    if type == 3:
        
        id  = int(line[1])
        n   = int(line[2])
        p   = int(line[3])
        
        cxy = {}
        d   = {}
        s   = {}
        w   = {}
        V   = [i for i in range(1,n+1)]
        
        for i in V:
            line    = file.readline().split()
            cxy[i]  = [int(line[0]), int(line[1])]
            s[i]    = int(line[2])
            w[i]    = int(line[3])
        
        for i in range(1,n+1):
            for j in range(i,n+1):
                e = Global.euclidean(cxy[i][0], cxy[j][0], cxy[i][1], cxy[j][1]) 
                d[i,j] = e
                d[j,i] = e

    

    #4 - OR-Library
    if type >= 4 and type <= 7:
        id = int(line[1])
        n  = int(line[2])
        p  = int(line[3])

        
        d  = {}
        s  = {}
        w  = {}
        V  = [i for i in range(1,n+1)]

        for i in V:
            j = 1
            for d_ in file.readline().split():
                d[i,j] = int(float(d_))
                j += 1

        i = 1
        for w_ in file.readline().split():
            w[i] = int(float(w_))
            i += 1
        
        i = 1
        for s_ in file.readline().split():
            s[i] = int(float(s_))
            i += 1
    
    file.close()

    return  id,n,p,d,s,w,cxy,V,id,type



from __future__ import print_function
from builtins import range
from pulp import *
import argparse
import importlib as ilib

try:
    from src.dippy import DipProblem, DipSolStatOptimal
    from src.dippy.examples.gen_func import *
except ImportError:
    from coinor.dippy import DipProblem, DipSolStatOptimal
    from coinor.dippy.examples.gen_func import *

def parseArgs():
    
    parser = argparse.ArgumentParser(
        description='Solve a traveling salesman problem.')
    parser.add_argument('--module', '-m', metavar = 'module name', 
                        help='name of the Python module from which to import data',
                        default = 'coinor.dippy.examples.tsp.tsp_data')

    addDippyArgs(parser)

    args = parser.parse_args()

    return(args)
 
def formulate(module_name):

    m = ilib.import_module(module_name)
    
    prob = DipProblem()

    arc_vars = LpVariable.dicts("UseArc", m.ARCS, 0, 1, LpBinary)

    # objective
    prob += lpSum(m.ARC_COSTS[x] * arc_vars[x] for x in m.ARCS)

    # degree constraints
    for city in m.CITIES:
        prob += lpSum(arc_vars[x] for x in m.CITY_ARCS[city]) \
                == 2

    # dictionary for symmetric arcs, can be
    # accessed using (i, j) and (j, i)
    symmetric = {}
    for i in m.CITIES:
        for j in m.CITIES:
            if i < j:
                symmetric[(i, j)] = (i, j)
                symmetric[(j, i)] = (i, j)

    prob.CITIES = m.CITIES
    prob.symmetric = symmetric
    prob.arc_vars = arc_vars

    return prob

# generate subtour elimination constraints
def generate_cuts(prob, sol):
    cons = []
    not_connected = set(prob.CITIES)

    while not_connected:
        start = not_connected.pop()
        nodes, arcs = get_subtour(prob, sol, start)
        if len(nodes) == len(arcs) and \
           len(nodes) < len(prob.CITIES):
            cons.append( lpSum(prob.arc_vars[a] for a in arcs) \
                         <= len(arcs) - 1 )
        nodes.remove(start)
        not_connected -= nodes

    return cons

def is_solution_feasible(prob, sol, tol):
    nodes, arcs = get_subtour(prob, sol, 0)

    return len(nodes) == len(arcs) and \
           len(nodes) == len(prob.CITIES)

def get_subtour(prob, sol, node):
    # returns: list of nodes and arcs
    # in subtour containing node
    nodes = set([node])
    arcs = set([])
    not_processed = set(prob.CITIES)
    to_process = set([node])

    tol = 1e-6
    one = 1 - tol

    while to_process:
        c = to_process.pop()
        not_processed.remove(c)
        new_arcs = [ prob.symmetric[(c, i)] \
                     for i in not_processed \
                     if sol[ \
                         prob.arc_vars[prob.symmetric[(c, i)]]]
                     > one]                     
        new_nodes = [ i for i in not_processed \
                      if prob.symmetric[(i, c)] in new_arcs ]
        arcs |= set(new_arcs)
        nodes |= set(new_nodes)
        to_process |= set(new_nodes)

    return nodes, arcs




from __future__ import absolute_import
import unittest
from pulp import *
import coinor.dippy as dippy

from .dippy_tests import DippyTestCase


class TestCokeProblem(DippyTestCase):

    def setUp(self):
        """
        sets up the coke problem as a unittest
        """
        self.prob, self.do_branch = create_coke_problem()

    def test_pulp_solve(self):
        """
        tests that pulp can solve the problem
        """
        status = self.prob.solve()
        self.assertEqual(status, LpStatusOptimal)
        self.assertAlmostEqual(self.prob.objective.value(), 191078860.725, 2)
        self.variable_feasibility_test(self.prob)
        self.constraint_feasibility_test(self.prob)

    def test_dippy_branch(self):
        """
        tests that the custom branch can solve the problem
        """
        self.prob.branch_method = self.do_branch
        dippy.Solve(self.prob, {})
        self.assertAlmostEqual(self.prob.objective.value(), 191078860.725, 2)
        self.variable_feasibility_test(self.prob)
        self.constraint_feasibility_test(self.prob)



def create_coke_problem():
    """
    creates and returns the coke problem
    """

    CC = 1.3
    BIG_M = 1e10

    MINE_SUPPLY = {
        "M1": 25.8,
        "M2": 728,
        "M3": 1456,
        "M4": 49,
        "M5": 36.9,
        "M6": 1100,
    }
    MINES = list(MINE_SUPPLY.keys())
    MINES.sort()

    LOCATIONS = ["L1", "L2", "L3", "L4", "L5", "L6"]

    SIZE_COSTS = {
        0: 0,
        75: 4.4,
        150: 7.4,
        225: 10.5,
        300: 13.5,
        375: 16.5,
        450: 19.6,
    }
    SIZES = list(SIZE_COSTS.keys())
    SIZES.sort()

    CUSTOMER_DEMAND = {
        "C1": 83,
        "C2": 5.5,
        "C3": 6.975,
        "C4": 5.5,
        "C5": 720.75,
        "C6": 5.5,
    }
    CUSTOMERS = list(CUSTOMER_DEMAND.keys())
    CUSTOMERS.sort()

    MINE_TRANSPORT_DATA = """
            L1      L2      L3      L4      L5      L6
    M1      231737  46813   79337   195845  103445  45186
    M2      179622  267996  117602  200298  128184  49046
    M3      45170   93159   156241  218655  103802  119616
    M4      149925  254305  76423   123534  151784  104081
    M5      152301  205126  24321   66187   195559  88979
    M6      223934  132391  51004   122329  222927  54357
    """

    CUST_TRANSPORT_DATA = """
            L1      L2      L3      L4      L5      L6
    C1      6736    42658   70414   45170   184679  111569
    C2      217266  227190  249640  203029  153531  117487
    C3      35936   28768   126316  2498    130317  74034
    C4      73446   52077   108368  75011   49827   62850
    C5      174664  177461  151589  153300  59916   135162
    C6      186302  189099  147026  164938  149836  286307
    """

    def read_table(data, coerce, transpose=False):
        lines = data.splitlines()
        headings = lines[1].split()
        result = {}
        for row in lines[2:]:
            items = row.split()
            for i, item in enumerate(items[1:]):
                if transpose: key = (headings[i], items[0])
                else: key = (items[0], headings[i])
                result[key] = coerce(item)
        return result

    MINE_TRANSPORT = read_table(MINE_TRANSPORT_DATA, int)
    CUST_TRANSPORT = read_table(CUST_TRANSPORT_DATA, int, \
                            transpose=True)

    #add both parts of the network together
    ARC_COSTS = MINE_TRANSPORT.copy()
    ARC_COSTS.update(CUST_TRANSPORT)
    ARCS = list(ARC_COSTS.keys())

    LOCATIONS_SIZES = [(l, s) for l in LOCATIONS
                        for s in SIZES]

    prob = dippy.DipProblem("Coke", LpMinimize)

    # create variables
    buildVars = LpVariable.dicts("Build", LOCATIONS_SIZES,
                                 cat = LpBinary)

    # create arcs
    flowVars = LpVariable.dicts("Arcs", ARCS, lowBound=0,
                                upBound = BIG_M)

    # objective
    prob += 1e6 * sum(buildVars[(l, s)] * SIZE_COSTS[s]
                      for (l, s) in LOCATIONS_SIZES) + \
            sum(flowVars[(s, d)] * ARC_COSTS[(s, d)]
                for (s, d) in ARCS), \
            "cost_of_building_and_transport"

    # plant availability
    for loc in LOCATIONS:
        prob += sum(flowVars[(loc, c)] for c in CUSTOMERS) \
                <= sum(buildVars[(loc, s)] * s
                       for s in SIZES), \
                       "Plant_%s_availability"%loc

    # one size
    for loc in LOCATIONS:
        prob += sum(buildVars[(loc, s)] for s in SIZES) == 1, \
                "Plant_%s_size"%loc

    # conserve flow (mines)
    # flows are in terms of tonnes of coke
    for m in MINES:
        prob += sum(flowVars[(m, j)] for j in LOCATIONS) \
                <= MINE_SUPPLY[m], "Supply_mine_%s"%m

    for loc in LOCATIONS:
        prob += sum(flowVars[(m, loc)] for m in MINES) - \
                CC * sum(flowVars[(loc, c)] for c in CUSTOMERS) \
                == 0, "Conserve_flow_location_%s"%loc

    for c in CUSTOMERS:
        prob += sum(flowVars[(loc, c)]  for loc in LOCATIONS) \
                >= CUSTOMER_DEMAND[c], "Demand_cust_%s"%c

    def do_branch(prob, sol):
        tol = 1e-10
        for loc in LOCATIONS:
            sol_size = sum(sol[buildVars[loc, size]] * \
                           size for size in SIZES)
            # determine if solsize is bigger than the largest
            # or smaller tham the smallest
            if (abs(sol_size - max(SIZES)) < tol
                    or abs(sol_size - min(SIZES))) < tol:
                continue
            #find the first one bigger or equal to sol_size
            bigger = min([s for s in SIZES
                         if s >= sol_size - tol])
            if bigger - sol_size > tol:
                down_branch_ub = dict([(buildVars[loc, s], 0)
                                       for s in SIZES
                                       if s <= sol_size])
                up_branch_ub = dict([(buildVars[loc, s], 0)
                                     for s in SIZES
                                     if s > sol_size])

                return {}, down_branch_ub, {}, up_branch_ub

    return prob, do_branch


if __name__ == '__main__':
    unittest.main()



"""
This is the testing framework used for the rest of the tests
"""
from __future__ import absolute_import
import unittest
from pulp import *
import coinor.dippy as dippy

class DippyTestCase(unittest.TestCase):

    def variable_feasibility_test(self, prob, tol = 1e-5):
        """
        tests that the problem is feasible for its class of variables
        """

        for var in prob.variables():
            self.assertTrue(var.valid(tol))
            self.assertLess(var.value(), var.upBound + tol)
            self.assertGreater(var.value(), var.lowBound - tol)
            if var.cat == LpInteger:
                self.assertAlmostEqual(int(var.value() + tol), var.value())

    def constraint_feasibility_test(self, prob, tol = 1e-5):
        """
        tests that the problem is feasible for its class of variables
        """

        for constraint_name, constraint in list(prob.constraints.items()):
            self.assertTrue(constraint.valid(tol))

if __name__ == '__main__':
    from .test_coke import *
    from .test_facility import *
    from .test_cutting_stock import *
    from .test_tsp import *
    unittest.main()



#!/usr/bin/env python

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import
from past.utils import old_div
from .mdbin_pack_func import MDBinPackProb, formulate, solve

if __name__ == '__main__':
    # Python starts here
    bpp = MDBinPackProb(ITEMS  = [11, 12, 13, 14, 21, 22],
                        LIMITS = ['CPU', 'RAM'],
                        volume = {(11, 'CPU'): 1,
                                  (12, 'CPU'): 1,
                                  (13, 'CPU'): 1,
                                  (14, 'CPU'): 1,
                                  (21, 'CPU'): 1,
                                  (22, 'CPU'): 1,
                                  (11, 'RAM'): 512,
                                  (12, 'RAM'): 512,
                                  (13, 'RAM'): 512,
                                  (14, 'RAM'): 512,
                                  (21, 'RAM'): 3072,
                                  (22, 'RAM'): 3072,
                                  },
                        capacity = {'CPU': 4, 'RAM': 4096})
  
    prob = formulate(bpp)
  
    prob.tol = pow(pow(2, -24), old_div(2.0, 3.0))
    xopt = solve(prob)
  
    if xopt is not None:
        for var in prob.variables():
            print(var.name, "=", xopt[var])
    else:
        print("Dippy could not find and optimal solution")    
  
    if prob.display_mode != 'off':
        numNodes = len(prob.Tree.get_node_list())
        if ((prob.Tree.attr['display'] == 'pygame') or 
            (prob.Tree.attr['display'] == 'xdot')):
            prob.Tree.display()

  



#!/usr/bin/env python

# Generalized Assignment Problem
# argument should be a problem file, see Dip/examples/GAP_Instance.cpp for format
# for an e.g. see gap0512-2.dat included in this directory

from __future__ import division
from __future__ import print_function
from builtins import str
from builtins import range
from past.utils import old_div
import sys

from pulp import LpVariable, LpBinary, lpSum, value, LpProblem, LpMaximize

try:
    from src.dippy import Solve
    from src.dippy.examples.gen_func import *
except ImportError:
    from coinor.dippy import Solve
    from coinor.dippy.examples.gen_func import *

from .gap_func import *

args = parseArgs()

# parse data file
prob = formulate(args.module)

dippyOpts = addDippyOpts(args)

dippyOpts['TolZero'] = '%s' % tol

Solve(prob, dippyOpts)

for m in prob.MACHINES:
    print() 
    print("Machine %d assigned tasks" %m, end=' ')
    for t in prob.TASKS:
        v = prob.assignVars[m][t].varValue
        if v:
            print("%d" %t, end=' ')
print()
            
if prob.display_mode != 'off':
    if (prob.Tree.attr['display'] == 'pygame') or (prob.Tree.attr['display'] == 'xdot'):
        prob.Tree.display()




from __future__ import print_function
from builtins import range
from pulp import *
import importlib as ilib

try:
    from src.dippy import DipProblem, DipSolStatOptimal
    from src.dippy.examples.gen_func import *
except ImportError:
    from coinor.dippy import DipProblem, DipSolStatOptimal
    from coinor.dippy.examples.gen_func import *

def parseArgs():
    parser = argparse.ArgumentParser(
        description='Solve a cutting stock problem.')
    parser.add_argument('--module', '-m', metavar = 'module name', 
                        help='name of the Python module from which to import data',
                        default = 'coinor.dippy.examples.csp.cutting_stock_data')
    parser.add_argument('--useCustomSolver', action='store_true', 
                        help='enable custom subproblem solver')

    addDippyArgs(parser)
    
    args = parser.parse_args()

    return(args)

def cross(i1, i2):
    r = []
    for a in i1:
        for b in i2:
            r.append((a, b))
    return r

def formulate(module_name):

    prob = DipProblem("Python", LpMinimize)

    m = ilib.import_module(module_name)
    
    # create variables
    useVars = LpVariable.dicts("Use", m.PATTERNS, 0, 1, LpBinary)
    prob.useVars = useVars

    cutVars = LpVariable.dicts("Cut", m.CUTS, 0, 10, LpInteger)
    prob.cutVars = cutVars

    # objective
    prob += lpSum(useVars[p] for p in m.PATTERNS), "min"

    # Meet demand
    for i in m.ITEMS:
        prob += lpSum(cutVars[(p, i)] for p in m.PATTERNS) \
                >= m.demand[i]

    # Ordering patterns
    for i, p in enumerate(m.PATTERNS):
        if p != m.PATTERNS[-1]:
            prob += useVars[p] >= useVars[m.PATTERNS[i+1]]

    for p in m.PATTERNS:
        prob.relaxation[p] += \
                lpSum(m.length[i] * cutVars[(p, i)] for i in m.ITEMS) \
                <= m.total_length[p] * useVars[p]

    prob.ITEMS = m.ITEMS
    prob.PATTERNS = m.PATTERNS
    
    return prob
        
def solve_subproblem(prob, keySub, redCosts, target):
    # get items with negative reduced cost
    item_idx = [i for i in ITEMS \
                if redCosts[cutVars[(keySub, i)]] < 0]
    vars = [cutVars[(keySub, i)] for i in item_idx]
    obj = [-redCosts[cutVars[(keySub, i)]] for i in item_idx]
    weights = [length[i] for i in item_idx]

    z, solution = kp(obj, weights, total_length[p])
    
    total_weight = sum(w * solution[i] \
                       for i, w in enumerate(weights))
    assert total_weight <= total_length[p]

    # add in reduced cost of useVars
    var_values = [(v, solution[i]) \
                  for i, v in enumerate(vars) \
                  if solution[i] > 0]
    var_values.append((useVars[keySub], 1))

    return DipSolStatOptimal, [var_values]

def kp(obj, weights, capacity):
    assert len(obj) == len(weights)
    n = len(obj)

    if n == 0:
        return 0, []

    if capacity == 0:
        return 0, [0 for i in range(n)]
    
    n = len(obj)

    # Don't include item
    zbest, solbest = kp(obj, weights, capacity - 1)
    # Check all items for inclusion
    for i in range(n):
        if weights[i] <= capacity:
            zyes, solyes = kp(obj, weights, \
                              capacity - weights[i])
            zyes += obj[i]
            solyes[i] += 1
            if zbest > zyes:
                zbest = zyes
                solbest = solyes

    return zbest, solbest




import argparse

def addDippyArgs(parser):
    parser.add_argument('--CGLCuts', action='store_true',
                        help='enable CGL cuts')
    parser.add_argument('--algo', choices=['Cut', 'PriceCut', 'Price'],
                        help='which algorithm to use', default = 'Cut') 
    parser.add_argument('--solveMasterAsMip', action='store_true',
                        help='enable solving the master an an MIP')
    parser.add_argument('--nodeLogInterval', type=int, metavar = 'N',
                        help='node logging interval') 
    parser.add_argument('--nodeLimit', type=int, metavar = 'L',
                        help='node limit')
    parser.add_argument('--enforceBranchInSubproblem', action='store_true',
                        help='enforce branching constraints in subproblem')

def addDippyOpts(args):

    dippyOpts = {}

    if args.algo == 'PriceCut':
        dippyOpts['doPriceCut'] = '1'
        dippyOpts['CutCGL'] = '1'
    elif args.algo == 'Price':
        dippyOpts['doPriceCut'] = '1'
        dippyOpts['CutCGL'] = '0'
    else:
        dippyOpts['doCut'] = '1'

    if args.CGLCuts:
        dippyOpts['CutCGL'] = '1'
    else:
        dippyOpts['CutCGL'] = '0'

    if args.solveMasterAsMip:
        dippyOpts['SolveMasterAsMip'] = 1
    if args.enforceBranchInSubproblem:
        dippyOpts['enforceBranchInSubproblem'] = 1
        
    if args.nodeLimit or args.nodeLogInterval:
        dippyOpts['ALPS'] = {}
        if args.nodeLimit:
            dippyOpts['ALPS']['nodeLimit'] = args.nodeLimit
        if args.nodeLogInterval:
            dippyOpts['ALPS']['nodeLogInterval'] = args.nodeLogInterval

    return dippyOpts



from __future__ import absolute_import
from builtins import range
import unittest
from pulp import *
import coinor.dippy as dippy

from .dippy_tests import DippyTestCase


class TestCuttingStockProblem(DippyTestCase):


    def test_pulp_solve(self):
        """
        tests that pulp can solve the problem
        """
        self.prob, self.relaxation = create_cutting_stock_problem()
        status = self.prob.solve()
        self.assertEqual(status, LpStatusOptimal)
        self.assertAlmostEqual(self.prob.objective.value(), 2.0)
        self.variable_feasibility_test(self.prob)
        self.constraint_feasibility_test(self.prob)

    def test_dippy_relaxation(self):
        """
        tests that the custom branch can solve the problem
        """
        self.prob, self.relaxation = create_cutting_stock_problem(doRelaxed=True)
        self.prob.relaxed_solver = self.relaxation
        dippy.Solve(self.prob, {
            'doPriceCut':1,
            'CutCGL': 0,
        })
        self.assertAlmostEqual(self.prob.objective.value(), 2.0)
        self.variable_feasibility_test(self.prob)
        self.constraint_feasibility_test(self.prob)


def create_cutting_stock_problem(doRelaxed=False):
    """
    creates and returns the cutting_stock problem
    """

    length = {
    "9cm": 9,
    "7cm": 7,
    "5cm": 5
    }

    ITEMS = list(length.keys())

    demand = {
    "9cm": 1,
    "7cm": 1,
    "5cm": 4
    }

    total_patterns = sum(demand[i] for i in ITEMS)

    total_length = {}
    for p in range(total_patterns):
        total_length[p] = 20
    PATTERNS = list(total_length.keys())

    CUTS = [(p, i) for p in PATTERNS
                   for i in ITEMS]

    prob = dippy.DipProblem("Sponge_Rolls", LpMinimize)

    # create variables

    useVars = LpVariable.dicts("Use", PATTERNS, 0, 1, LpBinary)

    cutVars = LpVariable.dicts("Cut", CUTS, 0, 10, LpInteger)

    # objective
    prob += sum(useVars[p] for p in PATTERNS), "min"

    # Meet demand
    for i in ITEMS:
        prob += sum(cutVars[(p, i)] for p in PATTERNS) \
                >= demand[i]

    # Ordering patterns
    for i, p in enumerate(PATTERNS):
        if p != PATTERNS[-1]:
             prob += useVars[p] >= useVars[PATTERNS[i+1]]

    # Cut patterns
    for p in PATTERNS:
        if doRelaxed:
            prob.relaxation[p] += sum(length[i] *
                                      cutVars[(p, i)] for i in ITEMS) \
                                      <= total_length[p] * useVars[p]
        else:
            prob += sum(length[i] * cutVars[(p, i)] for i in ITEMS) \
                 <= total_length[p] * useVars[p]


    def relaxed_solver(prob, patt, redCosts, convexDual):
    ##    print patt, "in", PATTERNS
    ##    print "redCosts =", redCosts
    ##    print "convexDual =", convexDual
        # get items with negative reduced cost
        item_idx = [i for i in ITEMS \
                    if redCosts[cutVars[(patt, i)]] < 0]
        vars = [cutVars[(patt, i)] for i in item_idx]
        obj = [-redCosts[cutVars[(patt, i)]] for i in item_idx]
        weights = [length[i] for i in item_idx]

    ##    print "Using knapsack heuristic"
    ##    print "item_idx =", item_idx
    ##    print "obj =", obj
    ##    print "weights =", weights
    ##    print "capacity =", total_length[patt]
        z, solution = kp(obj, weights, total_length[patt])
    ##    print "Number of items = ", len(item_idx)
    ##    for i in range(len(item_idx)):
    ##        print "Item ", item_idx[i], " has profit ", obj[i], " and weight ", weights[i]
    ##    print "Knapsack has capacity ", total_length[patt]
    ##    print "Value = ", z
    ##    for i in range(len(item_idx)):
    ##        print "Included[", item_idx[i], "] = ", solution[i]

        total_weight = sum(w * solution[i] \
                           for i, w in enumerate(weights))
        assert total_weight <= total_length[patt]

        # add in reduced cost of useVars
        totalCut = sum(solution)
    ##    print "z, redCosts[useVars[", patt, "]], convexDual", z, redCosts[useVars[patt]], convexDual
        if totalCut > 0:
            rc = -z + redCosts[useVars[patt]] - convexDual
        else:
            rc = -convexDual

    ##    print "rc =", rc
    ##    sys.stdout.flush()
        if rc < 0: # Using this pattern
            var_values = dict([(v, solution[i]) \
                               for i, v in enumerate(vars) \
                               if solution[i] > 0])
            if totalCut > 0:
                var_values[useVars[patt]] = 1
                cost = 1
            else:
                cost = 0

            var_tuple = (cost, rc, var_values)
            return [var_tuple]

        return []

    def kp(obj, weights, capacity):
        assert len(obj) == len(weights)
        n = len(obj)

        if n == 0:
            return 0, []

        if capacity == 0:
            return 0, [0 for i in range(n)]

        n = len(obj)

        # Don't include item
        zbest, solbest = kp(obj, weights, capacity - 1)
        # Check all items for inclusion
        for i in range(n):
            if weights[i] <= capacity:
                zyes, solyes = kp(obj, weights, \
                                  capacity - weights[i])
                zyes += obj[i]
                solyes[i] += 1
                if zyes > zbest:
                    zbest = zyes
                    solbest = solyes

        return zbest, solbest

    return prob, relaxed_solver


if __name__ == '__main__':
    unittest.main()



'''
Created on Dec 29, 2013

@author: ted
'''
from __future__ import division
from builtins import str
from builtins import range
from past.utils import old_div
import argparse
import random
from pulp import LpVariable, LpBinary, lpSum, value, LpProblem, LpMaximize

try:
    from src.dippy import DipProblem
    from src.dippy.examples.gen_func import *
except ImportError:
    from coinor.dippy import DipProblem
    from coinor.dippy.examples.gen_func import *

def parseArgs():
    parser = argparse.ArgumentParser(
        description='Generate and solve a random block-structured MILP.')
    parser.add_argument('--randomSeed', '-r', metavar = 'R', type=int,
                        help='a random seed', default = 2)
    parser.add_argument('--numBlocks', '-b', metavar = 'NB', type=int,
                        help='number of blocks', default = 2) 
    parser.add_argument('--numVarsPerBlock', '-v', metavar = 'NV', type=int,
                        help='number of variables per block', default = 10)
    parser.add_argument('--numConsPerBlock', '-c', metavar = 'NC', type=int,
                        help='number of constraints per block', default = 5)
    parser.add_argument('--numLinkingCons', '-l', metavar = 'NL', type=int,
                        help='number of linking constraints', default = 10)
    parser.add_argument('--tightness', '-t', metavar = 'T', type=int,
                        help='how tight the constraints should be (higher is tighter)',
                        default = 2)
    parser.add_argument('--density', '-d', metavar = 'D', type=int,
                        help='density of the constraint matrix', default = 0.2)
    parser.add_argument('--maxConsCoeff', '-m', metavar = 'M', type=int,
                        help='maximum size of a constraint coefficient',
                        default = 10)

    addDippyArgs(parser)
    
    args = parser.parse_args()

    return(args)
    
def GenerateRandomBlock(VARIABLES, CONSTRAINTS, density = 0.2,
                        maxObjCoeff = 10, maxConsCoeff = 10, 
                        tightness = 2, rand_seed = 2):
    random.seed(rand_seed)
    numCons = len(CONSTRAINTS)
    numVars = len(VARIABLES)
    OBJ = dict((i, random.randint(1, maxObjCoeff)) for i in VARIABLES)
    MAT = dict(((i, j), random.randint(1, maxConsCoeff) 
                        if random.random() <= density else 0)
                for j in CONSTRAINTS for i in VARIABLES)
    RHS = dict((i, random.randint(int(numVars*density*maxConsCoeff/tightness),
                                  int(numVars*density*maxConsCoeff/1.5)))
                                  for i in CONSTRAINTS)
    return OBJ, MAT, RHS

def formulate(args):

    prob = DipProblem("MILP")

    numBlocks = args.numBlocks
    numBlockVars = [args.numVarsPerBlock for i in range(numBlocks)]
    numBlockCons = [args.numConsPerBlock for j in range(numBlocks)]
    numVars = sum(numBlockVars)
    numCons = sum(numBlockCons) + args.numLinkingCons

    VARIABLES = dict(((i, j), 0) for i in range(numBlocks) 
                 for j in range(numBlockVars[i]))  

    CONSTRAINTS = []
    for k in range(numBlocks):
        CONSTRAINTS.append(["C"+str(k)+"_"+str(j) for j in range(numCons)])
    CONSTRAINTS.append(["C"+str(numBlocks)+"_"+str(j) for j in range(numCons)])

    #Generate random MILP
    var = LpVariable.dicts("x", VARIABLES, 0, 1, LpBinary)

    OBJ, MAT, RHS = GenerateRandomBlock(VARIABLES, CONSTRAINTS[numBlocks],
                                        rand_seed = args.randomSeed, tightness = args.tightness,
                                        density = args.density)

    prob += -lpSum([OBJ[i]*var[i] for i in var]), "Objective"

    #Linking constraints
    for j in CONSTRAINTS[numBlocks]:
        prob += lpSum([MAT[i, j]*var[i] for i in var]) <= RHS[j], j

    #Blocks    
    for k in range(numBlocks):
        OBJ, MAT, RHS = GenerateRandomBlock([(k, i) for i in range(numBlockVars[k])], 
                                        CONSTRAINTS[k])
        for j in CONSTRAINTS[k]:
            prob.relaxation[k] += (lpSum([MAT[(k, i), j]*var[k, i] for i in
                                          range(numBlockVars[k])]) <=RHS[j], j)
    return prob




"""
A formulation in original variables of a wedding seating problem

Authors: Stuart Mitchell 2010
"""
from __future__ import print_function
from builtins import range
import pulp
import argparse

try:
    from src.dippy import DipProblem, DipSolStatOptimal, Solve
    from src.dippy.examples.gen_func import *
except ImportError:
    from coinor.dippy import DipProblem, DipSolStatOptimal, Solve
    from coinor.dippy.examples.gen_func import *

debug_print = False

max_tables = 5
max_table_size = 4
guests = 'A B C D E F G I J K L M N O P Q R'.split()

def happiness(guest_a, guest_b):
    """
    Return the happiness (0 is the best) of allocating two
    guests together in the same table
    """
    return abs(ord(guest_a) - ord(guest_b))

parser = argparse.ArgumentParser(description='Solve a wedding planner problem.')
addDippyArgs(parser)
args = parser.parse_args()

#create the set of possible tables
tables = list(range(max_tables))

possible_seatings = [(g, t) for g in guests
                            for t in tables]

#create a binary variable to model if a guest sits at a particular table
x = pulp.LpVariable.dicts('possible_seatings', possible_seatings, 
                            lowBound = 0,
                            upBound = 1,
                            cat = pulp.LpInteger)

seating_model = DipProblem("Wedding Seating Model (DIP)", pulp.LpMinimize,
                           display_mode = 'off', display_interval = 10000)

#specify the maximum number of guests per table
for table in tables:
    seating_model.relaxation[table] += sum([x[(guest, table)]
                                            for guest in guests]) <= \
                                            max_table_size, \
                                            "Maximum_table_size_%s"%table

#A guest must seated at one and only one table
for guest in guests:
    seating_model += (sum([x[(guest, table)] for table in tables]) == 1, 
                      "Must_seat_%s"%guest)

#create a set of variables to model the objective function
possible_pairs = [(a, b) for a in guests for b in guests if ord(a) < ord(b)]
happy = pulp.LpVariable.dicts('table_happiness', tables,
                              lowBound = 0,
                              upBound = None,
                              cat = pulp.LpContinuous)

seating_model += sum([happy[table] for table in tables])

#create constraints for each possible pair
for table in tables:
    for (a, b) in possible_pairs:
        seating_model.relaxation[table] += \
            happy[table] >= (happiness(a, b) * (x[(a, table)] + 
                                                x[(b, table)] - 1))

def relaxed_solver(prob, table, redCosts, target):
    """
    Generate columns (tables) with negative reduced costs
    """
    dvs = []
    neg_guests = [g for g in guests
                       if redCosts[x[(g,table)]] < 0.0]
    neg_guests.sort()
    # find all possible tables between two end points
    for pos1, pos2 in [(i, j) for i in range(len(neg_guests))
                            for j in range(len(neg_guests))
                                if j > i]:
        # find the suitable guests that can be included in between the end 
        # points
        candidate_guests = [(redCosts[x[(g,table)]], g)
                                    for g in neg_guests[pos1+1:pos2]]
        candidate_guests.sort()
        # pick the best guests (ie those with the negative reduced costs)
        possible_table_inner = [g 
                            for _, g in candidate_guests[:max_table_size-2]]
        #This is the best table between the end points
        possible_table = [neg_guests[pos1]] + possible_table_inner +\
                            [neg_guests[pos2]]
        # calculate the sum of the reduced costs for each of the guests
        neg_cost = sum(redCosts[x[(g, table)]] for g in possible_table)
        table_happiness = happiness(possible_table[0], possible_table[-1])
        rc = neg_cost + table_happiness * redCosts[happy[table]]
        var_values = [(x[(g, table)], 1) 
                      for g in possible_table]
        var_values.append((happy[table], table_happiness))
        dvs.append(dict(var_values))
        if debug_print:
            print('Table: ', table, 'Happiness: ', table_happiness, 'RC: ', rc)
    return DipSolStatOptimal, dvs

#seating_model.relaxed_solver = relaxed_solver

#seating_model.writeLP('wedding_main.lp')
#for table in tables:
#    seating_model.writeRelaxed(table, 'wedding_relax%s.lp' % table);

dippyOpts = addDippyOpts(args)

Solve(seating_model, dippyOpts)

if seating_model.display_mode != 'off':
    numNodes = len(seating_model.Tree.get_node_list())
    if seating_model.Tree.attr['display'] == 'svg':
        seating_model.Tree.write_as_svg(filename = "facility_node%d" % (numNodes + 1), 
                               prevfile = "facility_node%d" % numNodes)
    seating_model.Tree.display()

for table in tables:
    print(table, end=' ')
    for guest in guests:
        if x[(guest,table)].value() >= 0.99:
            print(guest, end=' ')
    print(happy[table].value())




from .cutting_stock_func import cross

length = {
"9cm": 9,
"7cm": 7,
"5cm": 5
}

ITEMS = list(length.keys())

demand = {
"9cm": 3,
"7cm": 2,
"5cm": 2
}

total_patterns = sum(demand[i] for i in ITEMS)

total_length = {}
for p in range(total_patterns):
    total_length[p] = 20
PATTERNS = list(total_length.keys())

CUTS = cross(PATTERNS, ITEMS)




from math import sqrt

# 2d Euclidean TSP

# x,y coords of cities
CITY_LOCS = [(0, 2), (0, 4), (1, 2), (1, 4), \
             (4, 1), (4, 4), (4, 5),  (5, 0), \
             (5, 2), (5, 5)]
CITIES = list(range(len(CITY_LOCS)))

ARCS = [] # list of arcs (no duplicates)
ARC_COSTS = {} # distance

# for each city, list of arcs into/out of
CITY_ARCS = [[] for i in CITIES]

# use 2d euclidean distance
def dist(x1, y1, x2, y2):
    return sqrt((x1-x2)**2 + (y1-y2)**2)

# construct list of arcs
for i in CITIES:
    i_x, i_y = CITY_LOCS[i]
    for j in CITIES[i+1:]:
        j_x, j_y = CITY_LOCS[j]
        ARC_COSTS[(i,j)] = dist(i_x, i_y, j_x, j_y)
        ARCS.append((i, j))
        CITY_ARCS[i].append((i, j))
        CITY_ARCS[j].append((i, j))




# Generalized Assignment Problem
# argument should be a problem file, see Dip/examples/GAP_Instance.cpp for format
# for an e.g. see gap0512-2.dat included in this directory

from __future__ import division
from __future__ import print_function
from builtins import str
from builtins import range
from past.utils import old_div
import importlib as ilib
import argparse
from pulp import LpVariable, LpBinary, lpSum, value, LpProblem, LpMaximize

try:
    from src.dippy import DipProblem, DipSolStatOptimal
    from src.dippy.examples.gen_func import *
except ImportError:
    from coinor.dippy import DipProblem, DipSolStatOptimal
    from coinor.dippy.examples.gen_func import *

debug_print = False

tol = pow(pow(2, -24), old_div(2.0, 3.0))

def parseArgs():
    
    parser = argparse.ArgumentParser(
        description='Solve a generalized assignment problem.')
    parser.add_argument('--module', '-m', metavar = 'module name', 
                        help='name of the Python module from which to import data',
                        default = 'coinor.dippy.examples.gap.gap0515-2')

    addDippyArgs(parser)

    args = parser.parse_args()

    return args

def formulate(module_name):

    m = ilib.import_module(module_name)

    lines = m.gap_data.splitlines()
    line = lines[1].split() #first line is blank
    NUM_MACHINES = int(line[0])
    NUM_TASKS = int(line[1])
    MACHINES = list(range(NUM_MACHINES))
    TASKS = list(range(NUM_TASKS))
    MACHINES_TASKS = [(m, t) for m in MACHINES for t in TASKS]

    COSTS = []
    line_num = 2
    for m in MACHINES:
        line = lines[line_num].split()
        assert len(line) == NUM_TASKS
        COSTS.append([int(f) for f in line])
        line_num += 1

    RESOURCE_USE = []
    for m in MACHINES:
        line = lines[line_num].split()
        assert len(line) == NUM_TASKS
        RESOURCE_USE.append([int(f) for f in line])
        line_num += 1

    line = lines[line_num].split()
    assert len(line) == NUM_MACHINES
    CAPACITIES = [int(f) for f in line]

    assignVars = []
    for m in MACHINES:
        v = []
        for t in TASKS:
            v.append(LpVariable("M%dT%d" % (m, t), cat=LpBinary))
        assignVars.append(v)

    prob = DipProblem("GAP")

    # objective
    prob += lpSum(assignVars[m][t] * COSTS[m][t] for m, t in MACHINES_TASKS), "min"

    # machine capacity (knapsacks, relaxation)
    for m in MACHINES:
        prob.relaxation[m] += lpSum(assignVars[m][t] * RESOURCE_USE[m][t] for t in TASKS) <= CAPACITIES[m]

        # assignment
    for t in TASKS:
        prob += lpSum(assignVars[m][t] for m in MACHINES) == 1

    prob.assignVars = assignVars
    prob.MACHINES = MACHINES
    prob.TASKS = TASKS

    return prob

def solve_subproblem(prob, machine, redCosts, target):
    if debug_print:
        print("solve_subproblem...")
        print(redCosts)
   
    # get tasks which have negative reduced costs
    task_idx = [t for t in TASKS if redCosts[assignVars[machine][t]] < 0]
    var = [assignVars[machine][t] for t in task_idx]
    obj = [-redCosts[assignVars[machine][t]] for t in task_idx]
    weights = [RESOURCE_USE[machine][t] for t in task_idx]

    z, solution = knapsack01(obj, weights, CAPACITIES[machine])

    # Get the reduced cost of the knapsack solution and waste
    if debug_print:
        print([(v, redCosts[v]) for v in var])
        print(obj)
        print("z, solution =", z, solution)
        print("rc", -z)

    if z < -tol: # Zero solution is optimal
        if debug_print:
            print("Zero solution is optimal")
        return DipSolStatOptimal, [{}]

    var_values = dict([(var[i], 1) for i in solution])

    if debug_print:
        rcCheck = 0.0
        for v in list(var_values.keys()):
            rcCheck += redCosts[v] * var_values[v]
        print("Checking rc calc", -z, rcCheck) 
        print(var_values)
        
    return DipSolStatOptimal, [var_values]

def knapsack01(obj, weights, capacity):
    """ 0/1 knapsack solver, maximizes profit. weights and capacity integer """
        
    debug_subproblem = False
    
    assert len(obj) == len(weights)
    n = len(obj)
    if n == 0:
        return 0, []

    if (debug_subproblem):
        relaxation = LpProblem('relaxation', LpMaximize)
        relax_vars = [str(i) for i in range(n)]
        var_dict   = LpVariable.dicts("", relax_vars, 0, 1, LpBinary)
        relaxation += lpSum(var_dict[str(i)] * weights[i] for i in range(n)) <= capacity
        relaxation += lpSum(var_dict[str(i)] * obj[i] for i in range(n))
        relaxation.solve()
        relax_obj = value(relaxation.objective)

        solution =  [i for i in range(n) if var_dict[str(i)].varValue > tol ]

        print(relax_obj, solution)


    c = [[0]*(capacity+1) for i in range(n)]
    added = [[False]*(capacity+1) for i in range(n)]
    # c [items, remaining capacity]
    # important: this code assumes strictly positive objective values
    for i in range(n):
        for j in range(capacity+1):
            if (weights[i] > j):
                c[i][j] = c[i-1][j]
            else:
                c_add = obj[i] + c[i-1][j-weights[i]]
                if c_add > c[i-1][j]:
                    c[i][j] = c_add
                    added[i][j] = True
                else:
                    c[i][j] = c[i-1][j]

    # backtrack to find solution
    i = n-1
    j = capacity

    solution = []
    while i >= 0 and j >= 0:
        if added[i][j]:
            solution.append(i)
            j -= weights[i]
        i -= 1
        
    return c[n-1][capacity], solution




from __future__ import print_function
from __future__ import absolute_import
from builtins import str
from past.builtins import basestring
from builtins import object
import pulp

import sys

try:
    import path.py
except ImportError:
    pass

from .dipapi import DipAPI

from ._dippy import *

gimpy_installed = True
try:
    from src.gimpy import BinaryTree
except ImportError:
    try:
        from coinor.gimpy import BinaryTree
    except ImportError:
        gimpy_installed = False

if gimpy_installed:
    try:
        from coinor.grumpy import BBTree
    except ImportError:
        grumpy_installed = False
    else:
        grumpy_installed = True
else:
    grumpy_installed = False

class DipError(Exception):
    """
    Dip Exception
    """
	
# DIP solver status
#enum DecompSolverStatus {
#   DecompSolStatError,
#   DecompSolStatOptimal,
#   DecompSolStatFeasible,
#   DecompSolStatInfeasible,
#   DecompSolStatNoSolution
#};
DipSolStatError      = 0
DipSolStatOptimal    = 1
DipSolStatFeasible   = 2
DipSolStatInfeasible = 3
DipSolStatNoSolution = 4
DipStatus = {
             DipSolStatError:      "Error",
             DipSolStatOptimal:    "Optimal",
             DipSolStatFeasible:   "Feasible",
             DipSolStatInfeasible: "Infeasible",
             DipSolStatNoSolution: "No solution"
            }

_Solve = Solve
def Solve(prob, params=None):
    """
    Solve a DipProblem instance, returning a solution object

    @param prob A DipProblem instance to solve
    @param params A dictionary of parameters to pass to DIP
    """

    # params is a dictionary, keys are strings, values are
    # strings or dictionaries

    # if value is a dictionary, key is a section, and items
    # of dictionary are names/values

    # if value is a string, then section is NULL and key is name
    # for these parameters we also assign them to the 'DECOMP'
    # section as a convenience

    # the dictionary is converted into a dictionary of
    # strings indexed by (section, name) tuples

    processed = {}
    if params is None:
        params = {}

    if (prob.branch_method == None) and (prob.display_mode == 'off'):
        params['pyBranchMethod'] = '0'
    if (prob.post_process_node == None) and (prob.display_mode == 'off'):
        params['pyPostProcessNode'] = '0'
    if (prob.post_process_branch == None) and (prob.display_mode == 'off'):
        params['pyPostProcessBranch'] = '0'
    if prob.relaxed_solver == None:
        params['pyRelaxedSolver'] = '0'
    if prob.is_solution_feasible == None:
        params['pyIsSolutionFeasible'] = '0'
    
    if (prob.generate_cuts == None) and (prob.generate_cuts_from_node == None):
        params['pyGenerateCuts'] = '0'

    if prob.generate_cuts != None:
        prob.gen_cuts = True
    else:
        prob.gen_cuts = False
    if prob.generate_cuts_from_node != None:
        prob.gen_cuts_node = True
    else:
        prob.gen_cuts_node = False
    
    if prob.heuristics == None:
        params['pyHeuristics'] = '0'
    if prob.init_vars == None:
        params['pyInitVars'] = '0'
    if prob.is_solution_feasible == None:
        params['pyIsSolutionFeasible'] = '0'

    for key, value in list(params.items()):
        valid_types = (basestring, int, float)
        if not isinstance(key, basestring):
            raise DipError("Bad key in parameter dictionary, expecting string")
        if isinstance(value, dict):
            section = key
            for name, param_val in list(value.items()):
                if not isinstance(param_val, valid_types):
                    raise DipError("Bad value '%s' in parameter dictionary, expecting string or number" % param_val)
                processed[(section, name)] = str(param_val)
        elif isinstance(value, valid_types):
            # add this parameter to both the 'None' section and the 'DECOMP' section
            processed[(None, key)] = str(value)
            processed[('DECOMP', key)] = str(value)
        else:
            raise DipError("Bad value '%s' in parameter dictionary, expecting string" % value)

    # DIP only solves minimisation problems
    if prob.sense == pulp.LpMaximize:
        raise DipError("DIP assumes a minimize objective, but DipProblem has "+ 
                       "maximize objective.\n" +
                       "Use prob.sense = pulp.LpMinimize and prob.objective " +
                       "*= -1 to remedy")
    
    # DIP only allows non-negative variables. This is difficult
    # to transform automatically, so request re-formulation
    for v in prob.variables():
        if v.lowBound < 0:
            raise DipError("Variable %s has negative lower bound, please " +
                           "re-formulate using sum of non-negative variables" 
                           % v.name)
        
    # call the Solve method from _dippy
    try:
        status, message, solList, dualList = _Solve(prob, processed)
        # solList  is a list of (col_name, value) pairs
        # dualList is a list of (row_name, value) pairs

        if prob.display_mode == 'svg' and gimpy_installed:
            if prob.display_interval is not None:
                prob.Tree.write_as_svg(filename = "%s_%d" % (prob.svg_prefix, 
                                                             prob.last_svg + 1),
                                       prevfile = "%s_%d" % (prob.svg_prefix, 
                                                             prob.last_svg))
                prob.last_svg += 1
            
    except Exception as ex:
        print("Error returned from _dippy")
        print(ex)
        raise

    if solList is None:
        solution = None
    else:
        solDict = dict(solList)
        setVars = set(prob.variables())
        setSolVars = set(solDict.keys())
        diff = setVars.symmetric_difference(setSolVars)
        if len(diff) > 0:
            raise DipError("Solution and variable list don't match in " +
                           "dippy.Solve")

        solution = solDict
        for v in prob.variables():
            v.varValue = solution[v]
        
    if dualList is None:
        duals = None
    else:
        dualDict = dict([(c.getName(), v) for (c, v) in dualList])
        setCons = set(prob.constraints)
        setDualCons = set(dualDict.keys())
        diff = setCons.symmetric_difference(setDualCons)
        if len(diff) > 0:
            raise DipError("Duals and constraint list don't match in dippy.Solve")

        duals = dualDict

    # return status, message, solution and duals
    return status, message, solution, duals

def createBranchLabel(lbs, ubs):
    maxLabelWidth = 20
      
    both = set(lbs.keys()) & set(ubs.keys())
    lbOnly = set(lbs.keys()) - both
    ubOnly = set(ubs.keys()) - both
                        
    bothStr = ''
    first = True
    currWidth = len(bothStr)
    for l in both:
        addStr = str(int(lbs[l])) + '<=' + \
                 ''.join(x for x in str(l) 
                         if x not in '()_') \
                 + '<=' + str(int(ubs[l]))
        if first:
            newWidth = len(addStr) + currWidth
        else:
            newWidth = len(addStr) + currWidth + 2
        if newWidth <= maxLabelWidth:
            if first:
                first = False
            else:
                bothStr += ', '
                currWidth += 2
            bothStr += addStr
            currWidth += len(addStr)
        else:
            bothStr += '\n' + addStr
            currWidth = len(addStr)
#            bothStr += ' ...'
#            break

    lbOnlyStr = ''
    first = True
    currWidth = len(lbOnlyStr)
    for l in lbOnly:
        addStr = str(int(lbs[l])) + '<=' + \
                 ''.join(x for x in str(l) 
                         if x not in '()_')
        if first:
            newWidth = len(addStr) + currWidth
        else:
            newWidth = len(addStr) + currWidth + 2
        if newWidth <= maxLabelWidth:
            if first:
                first = False
            else:
                lbOnlyStr += ', '
                currWidth += 2
            lbOnlyStr += addStr
            currWidth += len(addStr)
        else:
            lbOnlyStr += '\n' + addStr
            currWidth = len(addStr)
#            lbOnlyStr += ' ...'
#            break

    ubOnlyStr = ''
    first = True
    currWidth = len(ubOnlyStr)
    for l in ubOnly:
        addStr = ''.join(x for x in str(l) 
                         if x not in '()_') \
                 + '<=' + str(int(ubs[l]))
        if first:
            newWidth = len(addStr) + currWidth
        else:
            newWidth = len(addStr) + currWidth + 2
        if newWidth <= maxLabelWidth:
            if first:
                first = False
            else:
                ubOnlyStr += ', '
                currWidth += 2
            ubOnlyStr += addStr
            currWidth += len(addStr)
        else:
            ubOnlyStr += '\n' + addStr
            currWidth = len(addStr)
#            ubOnlyStr += ' ...'
#            break

    labelStr = ''
    first = True
    if len(bothStr) > 0:
        labelStr += bothStr
        first = False
    if len(lbOnlyStr) > 0:
        if first:
            first = False
        else:
            labelStr += '\n'
        labelStr += lbOnlyStr
    if len(ubOnlyStr) > 0:
        if first:
            first = False
        else:
            labelStr += '\n'
        labelStr += ubOnlyStr
        
    return labelStr

import string
def asCplexName(name):
    #to remove illegal characters from the names
    trans = str.maketrans("-+[] ->/","________")
    
    return str(name).translate(trans)

class DipProblem(pulp.LpProblem, DipAPI):

    def __init__(self, *args, **kwargs):
        # callback functions can be passed to class constructor as keyword 
        # arguments
        self.branch_method = kwargs.pop('branch_method', None)
        self.post_process_branch = kwargs.pop('post_process_branch', None)
        self.post_process_node = kwargs.pop('post_process_node', None)
        self.relaxed_solver = kwargs.pop('relaxed_solver', None)
        self.is_solution_feasible = kwargs.pop('is_solution_feasible', None)
        self.generate_cuts = kwargs.pop('generate_cuts', None)
        self.generate_cuts_from_node = kwargs.pop('generate_cuts_from_node', 
                                                  None)
        self.heuristics = kwargs.pop('heuristics', None)
        self.init_vars = kwargs.pop('init_vars', None)
        self.display_mode = kwargs.pop('display_mode', 'off')
        self.display_interval = kwargs.pop('display_interval', 1)
        self.layout = kwargs.pop('layout', 'dot')
        self.svg_prefix = kwargs.pop('svg_prefix', 'tree')
        
        if self.display_mode != 'off':
            if not gimpy_installed:
                print("GiMPy not installed. Display mode set to 'off'")
                self.display_mode = 'off'
            else:
                if grumpy_installed:
                    self.Tree = BBTree()
                else:
                    if self.layout == 'bak':
                        print("GrUMPy not installed. Display mode set to 'off'")
                        self.display_mode = 'off'
                    else:
                        self.Tree = BinaryTree()
        if self.display_mode != 'off':
            self.Tree.set_display_mode(self.display_mode)
            self.Tree.set_layout(self.layout)

        super(DipProblem, self).__init__(*args, **kwargs)
        self._subproblem = []
        self.relaxation = RelaxationCollection(self)

    def deepcopy(self):
        # callback functions can be passed to class constructor as keyword 
        # arguments
        dipcopy = DipProblem(name = self.name, sense = self.sense)
        dipcopy.branch_method = self.branch_method
        dipcopy.is_solution_feasible = self.is_solution_feasible
        dipcopy.generate_cuts = self.generate_cuts
        dipcopy.heuristics = self.heuristics
        dipcopy.init_vars = self.init_vars

        # This code is taken from pulp.py and needs to be coordinated
        # with pulp.py to avoid errors
        if dipcopy.objective != None:
            dipcopy.objective = self.objective.copy()
        dipcopy.constraints = {}
        for k,v in self.constraints.items():
            dipcopy.constraints[k] = v.copy()
        dipcopy.sos1 = self.sos1.copy()
        dipcopy.sos2 = self.sos2.copy()

        dipcopy._subproblem = self._subproblem[:]
        for k in list(self.relaxation.keys()):
            dipcopy.relaxation[k] = self.relaxation[k].copy()

        return dipcopy
    
    def variables(self):
        """
        Returns a list of the problem variables
        Overrides LpProblem.variables()
        
        Inputs:
            - none
        
        Returns:
            - A list of the problem variables
        """
        variables = {}
        if self.objective:
            variables.update(self.objective)
        for c in self.constraints.values():
            variables.update(c)
        for p in sorted(self.relaxation.keys()):
            for c in self.relaxation[p].constraints.values():
                variables.update(c)
        variables = list(variables)
        variables = sorted(variables, key=lambda variable: variable.name)
        
        return variables

    def getObjective(self):
        """
        Return objective as a dictionary with LpVariables as keys
        and (non-zero) coefficients as values
        """
        return self.objective

    def getRows(self, problem=None):
        """
        Return constraints as a list of dictionaries with LpVariables as keys
        and (non-zero) coefficients as values. Constraints also have
        getName, getLb and getUb methods (i.e., a LpConstraint)

        problem = None implies the master problem, otherwise problem
        is a subproblem
        """

        if problem is None:
            problem = self

        for n, c in problem.constraints.items():
            if c.name == None:
                c.name = n
        constraints = list(problem.constraints.values())

        return constraints

    def getCols(self, problem=None):
        """
        Returns a list of variables. Variables have getName, getLb,
        getUb and isInteger methods

        problem = None implies the master problem, otherwise problem
        is a subproblem
        """

        if problem is None:
            variables = self.variables()
        else:
            variables = {}
            for c in problem.constraints.values():
                variables.update(c)
            variables = list(variables)
            variables = sorted(variables, key=lambda variable: variable.name)

        return variables

    def getRelaxsAsDict(self):
        """
        Returns the relaxation subproblems as a dictionary with keys as
        defined by the user and values as subproblems
        """
        return self.relaxation.dict

    def writeFull(self, instancefile, blockfile, mip = True):
        f = open(instancefile, "w")
        b = open(blockfile, "w")
        f.write("\\* "+self.name+" *\\\n")
        if self.sense == 1:
            f.write("Minimize\n")
        else:
            f.write("Maximize\n")
        wasNone, dummyVar = self.fixObjective()
        objName = self.objective.name
        if not objName: objName = "OBJ"
        f.write(self.objective.asCplexLpAffineExpression(objName, constant = 0))
        f.write("Subject To\n")
        b.write("NBLOCKS\n")
        b.write("%i\n" % len(self.relaxation.dict))
        for k in self.constraints:
            f.write(self.constraints[k].asCplexLpConstraint(k))
        blockId = 0
        for r in self.relaxation.dict:
            rname = asCplexName(str(r))
            b.write("BLOCK %d\n" % blockId) 
            for k in self.relaxation.dict[r].constraints:
                f.write(self.relaxation.dict[r].constraints[k].asCplexLpConstraint(str(k)+'_'+rname))
                b.write(str(k)+'_'+rname+'\n')
            blockId += 1
        vs = list(self.variables())
        # check if any names are longer than 100 characters
        long_names = [v.name for v in vs if len(v.name) > 100]
        if long_names:
            raise PulpError('Variable names too long for Lp format\n' 
                                + str(long_names))
        # check for repeated names
        repeated_names = {}
        for v in vs:
            repeated_names[v.name] = repeated_names.get(v.name, 0) + 1
        repeated_names = [(key, value) for key, value in list(repeated_names.items())
                            if value >= 2]
        if repeated_names:
            raise PulpError('Repeated variable names in Lp format\n' 
                                + str(repeated_names))
        # Bounds on non-"positive" variables
        # Note: XPRESS and CPLEX do not interpret integer variables without 
        # explicit bounds
        if mip:
            vg = [v for v in vs if not (v.isPositive() and v.cat == pulp.LpContinuous) \
                and not v.isBinary()]
        else:
            vg = [v for v in vs if not v.isPositive()]
        if vg:
            f.write("Bounds\n")
            for v in vg:
                f.write("%s\n" % v.asCplexLpVariable())
        # Integer non-binary variables
        if mip:
            vg = [v for v in vs if v.cat == pulp.LpInteger and not v.isBinary()]
            if vg:
                f.write("Generals\n")
                for v in vg: f.write("%s\n" % v.name)
            # Binary variables
            vg = [v for v in vs if v.isBinary()]
            if vg:
                f.write("Binaries\n")
                for v in vg: f.write("%s\n" % v.name)
        f.write("End\n")
        f.close()
        self.restoreObjective(wasNone, dummyVar)

    def writeRelaxed(self, block, filename, mip = True):
        """
        Write the given block into a .lp file.
        
        This function writes the specifications (NO objective function,
        constraints, variables) of the defined Lp problem to a file.
        
        Inputs:
            - block -- the key to the block to write
            - filename -- the name of the file to be created.          
                
        Side Effects:
            - The file is created.
        """
        relaxation = self.relaxation[block]
        f = open(filename, "w")
        f.write("\\* "+relaxation.name+" *\\\n")
        f.write("Subject To\n")
        ks = list(relaxation.constraints.keys())
        ks.sort()
        for k in ks:
            f.write(relaxation.constraints[k].asCplexLpConstraint(k))
        vs = relaxation.variables()
        # check for repeated names
        relaxation.checkDuplicateVars()
        # Bounds on non-"positive" variables
        # Note: XPRESS and CPLEX do not interpret integer variables without 
        # explicit bounds
        if mip:
            vg = [v for v in vs if not (v.isPositive() and \
                                        v.cat == pulp.LpContinuous) \
                and not v.isBinary()]
        else:
            vg = [v for v in vs if not v.isPositive()]
        if vg:
            f.write("Bounds\n")
            for v in vg:
                f.write("%s\n" % v.asCplexLpVariable())
        # Integer non-binary variables
        if mip:
            vg = [v for v in vs if v.cat == pulp.LpInteger and \
                                   not v.isBinary()]
            if vg:
                f.write("Generals\n")
                for v in vg: f.write("%s\n" % v.name)
            # Binary variables
            vg = [v for v in vs if v.isBinary()]
            if vg:
                f.write("Binaries\n")
                for v in vg: f.write("%s\n" % v.name)
        f.write("End\n")
        f.close()
        
    def chooseBranchSet(self, xhat):
        """
        Finds the best branch for a fractional solution

        Inputs:
        xhat (list of (LpVariable, value) tuples) = list of solution values for all variables

        Output:
        down_lb, down_ub, up_lb, up_ub (tuple of (LpVariable, value) dictionaries) =
        lower and upper bounds for down branch, lower and upper bounds for up branch
        """
        try:
            if self.branch_method is None:
                return None
                        
            xhatDict = dict(xhat)
            setVars = set(self.variables())
            setXhatVars = set(xhatDict.keys())
            diff = setVars.symmetric_difference(setXhatVars)
            if len(diff) > 0:
                raise DipError("Solution and variable list don't match in chooseBranchSet")
    
            branch_sets = self.branch_method(self, xhatDict)

            if branch_sets is None:
                return None
                            
            if (branch_sets[0] or branch_sets[1]) and (branch_sets[2] or branch_sets[3]):
                return branch_sets
            else:
                raise DipError("Invalid bounds returned from user-specified branch_method")
        except Exception as ex:
            errorStr = "Error in chooseBranchSet\n%s" % ex
            raise DipError(errorStr)
          
    def decipherNode(self, output):
        outputDict = dict(output)
        if "xhat" in list(outputDict.keys()):
            xhat = outputDict["xhat"]
            outputDict["xhat"] = dict(xhat)
        if "bounds" in list(outputDict.keys()):
            bounds = outputDict["bounds"]
            outputDict["bounds"] = dict(bounds)
            
        return outputDict

    def postProcessNode(self, node):
        """
        Returns information from the node that has just been processed.

        Inputs:
        output (list of (parameter, value) tuples) = list of output values 
        from the node
        """
        try:        
            nodeDict = self.decipherNode(node)
            
            if gimpy_installed:
                nodeInd = nodeDict["nodeIndex"]
                parentInd = nodeDict["parentIndex"]
                nodeQuality = nodeDict["nodeQuality"]
                branchedDir = nodeDict["branchedDir"]
                nodeStatus = nodeDict["nodeStatus"]
                if branchedDir == -1:
                    branch_direction = 'L'
                else:
                    branch_direction = 'R'
                    
                if nodeStatus == 'Infeasible':
                    status = 'I'
                    BAKstatus = 'infeasible'
                    color = 'orange'
                elif nodeStatus == 'Candidate':
                    status = 'C'
                    BAKstatus = 'candidate'
                    color = 'yellow'
                elif nodeStatus == 'Solution':
                    status = 'S'
                    BAKstatus = 'integer'
                    color = 'lightblue'
                else:
                    status = 'P'
                    BAKstatus = 'fathomed'
                    color = 'red'
                    
                if nodeStatus != 'Infeasible':
                    label = status + ": " + "%.1f"%nodeQuality
                else:
                    label = 'I'

                numNodes = len(self.Tree.get_node_list())
                if parentInd == -1:
                    if self.layout == 'bak':
                        self.Tree.AddOrUpdateNode(nodeInd, parentInd,
                                                  branch_direction, BAKstatus, 
                                                  nodeQuality, None, None)
                    else:
                        self.Tree.add_root(nodeInd, label = label, 
                                           status = 'C', obj = nodeQuality, 
                                           color = color, style = 'filled', 
                                           fillcolor = color)
                    if self.Tree.attr['display'] == 'svg':
                        if self.display_interval is not None:
                            if numNodes % self.display_interval in [0, 1]:
                                self.Tree.write_as_svg(filename = "%s_0" 
                                                       % self.svg_prefix, 
                                                       nextfile = "%s_1" 
                                                       % self.svg_prefix, 
                                                       highlight = nodeInd)
                                self.last_svg = 0
                    numNodes += 1
                else:
                    if branch_direction == 'L':
                        n = self.Tree.get_left_child(parentInd)
                    else:
                        n = self.Tree.get_right_child(parentInd)
                    edge_label = self.Tree.get_edge_attr(parentInd, n, 'label')
                    self.Tree.del_node(n)
                    if self.layout == 'bak':
                        self.Tree.AddOrUpdateNode(nodeInd, parentInd, 
                                                  branch_direction, 'branched', 
                                                  nodeQuality, None, None)
                    elif branch_direction == 'L':
                        self.Tree.add_left_child(nodeInd, parentInd, 
                                                 label = label, 
                                                 status = status, 
                                                 obj = nodeQuality, 
                                                 color = color, 
                                                 style = 'filled', 
                                                 fillcolor = color)
                    else:
                        self.Tree.add_right_child(nodeInd, parentInd, 
                                                  label = label, 
                                                  status = status, 
                                                  obj = nodeQuality, 
                                                  color = color, 
                                                  style = 'filled', 
                                                  fillcolor = color)
                    if edge_label is not None:
                        self.Tree.set_edge_attr(parentInd, nodeInd, 
                                                'label', edge_label)
                    if self.Tree.attr['display'] == 'svg':
                        if self.display_interval is not None:
                            if numNodes % self.display_interval in [0, 1]:
                                self.Tree.write_as_svg(filename = "%s_%d" 
                                                       % (self.svg_prefix, 
                                                          self.last_svg + 1), 
                                                       prevfile = "%s_%d" 
                                                       % (self.svg_prefix, 
                                                          self.last_svg), 
                                                       nextfile = "%s_%d" 
                                                       % (self.svg_prefix, 
                                                          self.last_svg + 2), 
                                                       highlight = nodeInd)
                                self.last_svg += 1
                if self.display_interval is not None:
                    if numNodes % self.display_interval in [0, 1]:
                        self.Tree.display()

            if self.post_process_node is not None:
                self.post_process_node(self, nodeDict)
          
        except Exception as ex:
            errorStr = "Error in postProcessNode\n%s" % ex
            raise DipError(errorStr)

    def postProcessBranch(self, branchInfo):
        """
        Returns information from the node that has just been processed.

        Inputs:
        output (list of (parameter, value) tuples) describing branching decision
        """
        try:
          
            outputDict = dict(branchInfo)

            if gimpy_installed:
                nodeInd = outputDict['nodeIndex']
                nodeQuality = outputDict['nodeQuality']
                numNodes = len(self.Tree.get_node_list())
                for n in outputDict:
                    if n == 'pDownUB':
                        if self.layout == 'bak':
                            self.Tree.AddOrUpdateNode(-numNodes, 
                                                      nodeInd, 'L', 
                                                      'candidate', 
                                                      nodeQuality, None, None)
                        else:
                            self.Tree.add_left_child(-numNodes, 
                                                     nodeInd, 
                                                     label = 'C', 
                                                     status = 'C', 
                                                     obj = nodeQuality, 
                                                     color = 'yellow', 
                                                     style = 'filled', 
                                                     fillcolor = 'yellow')
                        if 'pDownLB' in outputDict:
                            lbs = outputDict['pDownLB']
                        else:
                            lbs = {}
                        ubs = outputDict['pDownUB']
                        labelStr = createBranchLabel(lbs, ubs)
                        self.Tree.set_edge_attr(nodeInd, 
                                                -numNodes, 
                                                'label', labelStr)
                        numNodes += 1

                    elif n == 'pUpLB':
                        if self.layout == 'bak':
                            self.Tree.AddOrUpdateNode(-numNodes, 
                                                      nodeInd, 'R', 
                                                      'candidate', 
                                                      nodeQuality, None, None)
                        else:
                            self.Tree.add_right_child(-numNodes, 
                                                      nodeInd, 
                                                      label = 'C', 
                                                      status = 'C', 
                                                      obj = nodeQuality, 
                                                      color = 'yellow', 
                                                      style = 'filled', 
                                                      fillcolor = 'yellow')
                        if 'pUpUB' in outputDict:
                            ubs = outputDict['pUpUB']
                        else:
                            ubs = {}
                        lbs = outputDict['pUpLB']
                        labelStr = createBranchLabel(lbs, ubs)
                        self.Tree.set_edge_attr(nodeInd, 
                                                -numNodes, 
                                                'label', labelStr)
                        numNodes += 1

                if self.Tree.get_node_attr(nodeInd, 'color') == 'yellow':
                    self.Tree.set_node_attr(nodeInd, 'color', 'green')
                    self.Tree.set_node_attr(nodeInd, 'fillcolor', 'green')
        
            if self.post_process_branch is not None:
                self.post_process_branch(self, outputDict)
  
        except Exception as ex:
            errorStr = "Error in postProcessBranch\n%s" % ex
            raise DipError(errorStr)

    def solveRelaxed(self, key, redCostX, target):
        """
        Returns solutions to the whichBlock relaxed subproblem
    
        Inputs:  
        key (Python Object) = key of relaxed subproblem to be solved
        redCostX (list of (variable, value) tuples) = list of reduced costs for all variables
        target (float) = any total reduced cost less than the target is "good" (results in a negative cost column)
    
        Output:
        status (integer) = status of the relaxation solve, will be one of
          DipSolStatOptimal    = no better columns can be found
          DipSolStatFeasible   = better columns can be found, but just use these for now
          DipSolStatNoSolution = use any columns returned, but also use DIP's default column finder
        varList (list of (cost, reduced cost, list of (variable, value) dictionaries)) =
        solution for this relaxed subproblem expressed as a cost, reduced cost and
        dictionary of non-zero values for variables
        """
        try:
          
            # transform redCostX into a dictionary
            redCostDict = dict(redCostX)
            setVars = set(self.variables())
            setRedCostVars = set(redCostDict.keys())
            diff = setVars.symmetric_difference(setRedCostVars)
            if len(diff) > 0:
                print(diff)
                raise DipError("Reduced cost and variable list don't match in",
                               "solveRelaxed")
    
            status, dvs = self.relaxed_solver(self, key, redCostDict, target)
    
            if len(dvs) > 0:
                dvs_with_costs = []
                for var in dvs:
                    if isinstance(var, dict):
                        cost = sum(self.objective[i]*var[i] for i in var
                                   if i in self.objective)
                        red_cost = sum(redCostDict[i]*var[i] for i in var
                                       if i in redCostDict)
                        dvs_with_costs.append((cost, red_cost, var))
                    else:
                        return status, dvs
                return status, dvs_with_costs
            else:
                return status, dvs
  
        except Exception as ex:
            errorStr = "Error in solveRelaxed\n%s" % ex
            raise DipError(errorStr)

    def isUserFeasible(self, sol, tol):
        """
        Lets the user decide if an integer solution is really feasible

        Inputs:
        sol (list of (LpVariable, value) tuples) = list of solution
        values for all variables tol (double) = zero tolerance

        Outputs:
        (boolean) = false if not feasible (generate cuts) or true if feasible
        """
        try:
              
            solDict = dict(sol)
            setVars = set(self.variables())
            setSolVars = set(solDict.keys())
            diff = setVars.symmetric_difference(setSolVars)
            if len(diff) > 0:
                raise DipError("Solution and variable list don't match in isUserFeasible")
    
            return self.is_solution_feasible(self, solDict, tol)
          
        except Exception as ex:
            errorStr = "Error in isUserFeasible\n%s" % ex
            raise DipError(errorStr)

    def generateCuts(self, node):
        """
        Lets the user generate cuts to remove fractional "pieces" of xhat

        Inputs:
        node (list of (string, object) tuples) = list of node properties

        Output:
        cutList (list of LpConstraints) =
        cuts for this fractional solution expressed as a list LpConstraints,
        i.e., a dictionary with LpVariables as keys and (non-zero) coefficients
        as values with getName, getLb and getUb bound methods
        """
        try:
    
            nodeDict = self.decipherNode(node)
            xhatDict = nodeDict["xhat"]
            setVars = set(self.variables())
            setXhatVars = set(xhatDict.keys())
            diff = setVars.symmetric_difference(setXhatVars)
            if len(diff) > 0:
                raise DipError("Solution and variable list don't match in generateCuts")
    
            # Generate a list of cuts as LpConstraints
            if self.gen_cuts:
                cuts = self.generate_cuts(self, xhatDict)
            else:
                cuts = None
    
            if self.gen_cuts_node:
                moreCuts = self.generate_cuts_from_node(self, nodeDict)
                if moreCuts is not None:
                    if cuts is None:
                        cuts = moreCuts
                else:
                    cuts.extend(moreCuts)
                    
            if cuts is not None:
                if len(cuts) > 0:
                    return cuts
                else:
                    print("Empty cut list in generateCuts, returning None")

        except Exception as ex:
            errorStr = "Error in generateCuts\n%s" % ex
            raise DipError(errorStr)

    def solveHeuristics(self, xhat, costX):
        """
        Lets the user generate (heuristic) solutions from a fractional solution

        Inputs:
        xhat  (list of (LpVariable, value) tuples) = list of solution values for all variables
        costX (list of (LpVariable, value) tuples) = list of costs for all variables

        Outputs:
        solList (list of (LpVariable, value) dictionaries) =
        solutions found from this fractional solution expressed as a
        dictionary of non-zero values for variables
        """
        try:
            
            # transform xhat into a dictionary
            xhatDict = dict(xhat)
            setVars = set(self.variables())
            setXhatVars = set(xhatDict.keys())
            diff = setVars.symmetric_difference(setXhatVars)
            if len(diff) > 0:
                raise DipError("Solution and variable list don't match in solveHeuristics")
    
            # transform costs into a dictionary
            costDict = dict(costX)
            setCostVars = set(costDict.keys())
            diff = setVars.symmetric_difference(setCostVars)
            if len(diff) > 0:
                raise DipError("Cost and variable list don't match in solveHeuristics")
    
            sols = self.heuristics(self, xhatDict, costDict)
            if sols is not None:
                if len(sols) > 0:
                    return sols
                else:
                    print("Empty solution list in solveHeuristics, returning None")

        except Exception as ex:
            errorStr = "Error in solveHeuristics\n%s" % ex
            raise DipError(errorStr)

    def generateInitVars(self):
        """
        Returns initial solutions to relaxed subproblems

        Inputs:
        None

        Output:
        varList (list of (subproblem key, (cost, (LpVariable, value) dictionaries))) =
        initial solutions for the relaxed subproblems expressed as a cost and
        dictionary of non-zero values for variables
        """
        try:
          
            bvs = self.init_vars(self)
            if bvs is not None:
                if len(bvs) > 0:
                    return bvs
                else:
                    print("Empty variable list in generateInitVars, returning None")

        except Exception as ex:
            errorStr = "Error in generateInitVars\n%s" % ex
            raise DipError(errorStr)
  

class RelaxationCollection(object):
    """
    A simple defaultdict for holding relaxation problems
    """
    PROBLEM_CLASS = pulp.LpProblem


    def __init__(self, parent):
        self.parent = parent
        self.dict = {}

    def __getitem__(self, name):
        if name not in self.dict:
            self.dict[name] = self.PROBLEM_CLASS()
        return self.dict[name]

    def __setitem__(self, name, value):
        self.dict[name] = value

    def keys(self):
        return list(self.dict.keys())

    def values(self):
        return list(self.dict.values())



#!/usr/bin/env python

# bin_pack_instance.py
from __future__ import division
from __future__ import print_function
from __future__ import absolute_import
from past.utils import old_div
from .bin_pack_func import BinPackProb, formulate, solve

import sys

if __name__ == '__main__':
    # Python starts here
    bpp = BinPackProb(ITEMS  = [1, 2, 3, 4, 5],
                      volume = {1: 2, 2: 5, 3: 3, 4: 7, 5: 2},
                      capacity = 8)
  
    prob = formulate(bpp)

    # Set a zero tolerance (Mike Saunders' "magic number")	
    prob.tol = pow(pow(2, -24), old_div(2.0, 3.0))
    if len(sys.argv) > 1:
        xopt = solve(prob, sys.argv[1])
    else:
        xopt = solve(prob)
  
    if xopt is not None:
        for var in prob.variables():
            print(var.name, "=", xopt[var])
    else:
        print("Dippy could not find and optimal solution")    
  
    if prob.display_mode != 'off':
        numNodes = len(prob.Tree.get_node_list())
        if ((prob.Tree.attr['display'] == 'pygame') or 
            (prob.Tree.attr['display'] == 'xdot')):
            prob.Tree.display()



#!/usr/bin/env python

from __future__ import division
from builtins import range
from past.utils import old_div
__title__     = 'B&P-cut for the Capacitated Vertex p-Median Problem (CVPMP)'
__version__   = '1.0 Nov 2013'
__author__    = 'Dago Quevedo'
__email__     = 'dago@yalma.fime.uanl.mx'

from pulp import LpVariable, LpBinary, lpSum, value, LpProblem, LpMaximize
from math import *
from inspect import getfile
from os.path import join, dirname
import coinor.dippy.examples.cvpmp
try:
    from src.dippy import DipProblem, Solve, DipSolStatOptimal
    from src.dippy.examples.gen_func import *
except ImportError:
    from coinor.dippy import DipProblem, Solve, DipSolStatOptimal
    from coinor.dippy.examples.gen_func import *

#Globar vars

n  = None
p  = None
d  = None
s  = None
w  = None
V  = None
x  = None
y  = None

tol          = pow(pow(2, -24), old_div(2.0, 3.0))
display_mode = 'off'

def parseArgs():
    
    parser = argparse.ArgumentParser(
        description='Solve a p-median problem.')
    parser.add_argument('--instance', metavar = 'file', 
                        help='path to the file from which to import data',
                        default = join(dirname(getfile(coinor.dippy.examples.cvpmp)),
                                       'Instances','pmedcap1.dat'))
    parser.add_argument('--useCustomSolver', action='store_true', 
                        help='enable custom subproblem solver')

    addDippyArgs(parser)

    args = parser.parse_args()

    return args

def Init(_n,_p,_d,_s,_w,_V):
    
    global n, p, d, s, w, V
    n   = _n
    p   = _p
    d   = _d
    s   = _s
    w   = _w
    V   = _V

def solve_subproblem(prob, i, redCosts, target):
    
    vars   = [x[(i, j)] for j in V]
    obj    = [max(-redCosts[x[(i, j)]], 0) for j in V]
    weights= [w[j] for j in V]

    #Solver a knapsack for i
    z, solution = KP01(obj, weights, s[i])
    rc = redCosts[y[i]] - z
    
    #Cost
    cost           = sum([d[i,V[j]] for j in solution])

    #Cluster of customers for i
    var_val        = dict([(vars[j], 1) for j in solution])
    var_val[y[i]]  = 1

    return DipSolStatOptimal, [var_val]

def KP01(obj, weights, capacity):
    
    assert len(obj) == len(weights)

    n = len(obj)
    if n == 0:
        return 0, []
    
    c     = [[0]*(capacity+1) for i in range(n)]
    added = [[False]*(capacity+1) for i in range(n)]
    
    for i in range(n):
        for j in range(capacity + 1):
            if (weights[i] > j):
                c[i][j] = c[i-1][j]
            else:
                c_add = obj[i] + c[i-1][j-weights[i]]
                if c_add > c[i-1][j]:
                    c[i][j]     = c_add
                    added[i][j] = True
                else:
                    c[i][j] = c[i-1][j]
    
    i = n-1
    j = capacity
    
    solution = []
    while i >= 0 and j >= 0:
        if added[i][j]:
            solution.append(i)
            j -= weights[i]
        i -= 1
    
    return c[n-1][capacity], solution



def Solver(args):
    
    global  x,y

    prob = DipProblem("CVPMP", display_mode = display_mode,
                      layout = 'dot', display_interval = 0)
    
    X = [(i, j) for i in V for j in V]
    x = LpVariable.dicts("x", X, 0, 1, LpBinary)
    y = LpVariable.dicts("y", V, 0, 1, LpBinary)

    prob += (lpSum(d[i, j] * x[(i, j)] for i in V for j in V), "min")
    
    #linking constraints
    for j in V:
        prob += lpSum(x[(i,j)] for i in V) == 1
    
    #non-relaxing
    for i in V:
        prob.relaxation[i] += lpSum(w[j] * x[(i, j)]
                                    for j in V) <= s[i]*y[i]

    prob += lpSum(y[i] for i in V) == p

    if args.useCustomSolver:
        prob.relaxed_solver = solve_subproblem

    dippyOpts = addDippyOpts(args)
    
    Solve(prob, dippyOpts)

    #Make solution
    solution = []
    for i in V:
        if y[i].varValue:
            cluster = []
            for j in V:
                if x[(i, j)].varValue:
                    cluster.append(j)

            solution.append((i,cluster))


    return round(prob.objective.value()), solution




from builtins import range
from builtins import object
import sys

from pulp import LpVariable, lpSum, LpBinary, LpStatusOptimal

try:
    import path
except ImportError:
    pass
        
try:
    import dippy
except ImportError:
    try:
        import src.dippy as dippy
    except ImportError:
        import coinor.dippy as dippy

from math import floor, ceil

class MDBinPackProb(object):
    def __init__(self, ITEMS, LIMITS, volume, capacity):
        self.ITEMS = ITEMS
        self.LIMITS = LIMITS
        self.volume = volume
        self.capacity = capacity

        self.BINS = list(range(len(ITEMS))) # Create 1 bin for each item, indices 
                                      # start at 0
    
def formulate(bpp):
    prob = dippy.DipProblem("Bin Packing",
                            display_mode = 'xdot',
#                           layout = 'bak',
                            display_interval = None,
                            )

    assign_vars = LpVariable.dicts("x",
                                   [(i, j) for i in bpp.ITEMS
                                    for j in bpp.BINS],
                                   cat=LpBinary)
    use_vars    = LpVariable.dicts("y", bpp.BINS, cat=LpBinary)
    waste_vars  = LpVariable.dicts("w", [(j, k) for j in bpp.BINS
                                          for k in bpp.LIMITS], 0, None)

    prob += lpSum(use_vars[j] for j in bpp.BINS), "min_bins"

    for j in bpp.BINS:
      for k in bpp.LIMITS:
        prob += lpSum(bpp.volume[i, k] * assign_vars[i, j] for i in bpp.ITEMS) \
                + waste_vars[j, k] == bpp.capacity[k] * use_vars[j]

    for i in bpp.ITEMS:
        prob += lpSum(assign_vars[i, j] for j in bpp.BINS) == 1

    for i in bpp.ITEMS:
        for j in bpp.BINS:
            prob += assign_vars[i, j] <= use_vars[j]

    for n in range(0, len(bpp.BINS) - 1):
        prob += use_vars[bpp.BINS[n]] >= use_vars[bpp.BINS[n + 1]]

    # Attach the problem data and variable dictionaries to the DipProblem 
    prob.bpp         = bpp
    prob.assign_vars = assign_vars
    prob.use_vars    = use_vars
    prob.waste_vars  = waste_vars

    return prob

def solve(prob):
 
    dippyOpts = {
#                 'doPriceCut' : '1',
                 'CutCGL': '1',
#                'SolveMasterAsIp': '0'
#                'generateInitVars': '1',
#                 'LogDebugLevel': 5,
#                'LogDumpModel': 5,
                 }

    status, message, primals, duals = dippy.Solve(prob, dippyOpts)
  
    if status == LpStatusOptimal:
        return dict((var, var.value()) for var in prob.variables())
    else:
        return None
         



#!/usr/bin/env python


__title__     = 'General tools'
__version__   = '1.0 Nov 2013'
__author__    = 'Dago Quevedo'
__email__     = 'dago@yalma.fime.uanl.mx'


import math
import os
import sys
import time
#import resource


def euclidean(x1, x2, y1, y2):
    return int(math.sqrt(math.pow(x1 - x2, 2)+math.pow(y1 - y2, 2)))

def indexOf(L, value):
    try:
        i = L.index(value)
    except ValueError:
        i = -1
    
    return i

def path():
    return os.path.abspath(os.path.split(sys.argv[0])[0])


#!/usr/bin/env python

from setuptools import setup, Extension, find_packages
import subprocess, os, sys
from os.path import join, dirname
from os import listdir

PROJECT = 'coinor.dippy'
VERSION = '1.95.2'
URL = 'https://github.com/coin-or/Dip'
DESC = u'DIP Python Interface'

def read_file(file_name):

    file_path = join(
        dirname(__file__),
        file_name
        )

    return open(file_path).read()

try:
    CoinDir = os.environ['COIN_INSTALL_DIR']
except:
    from os.path import abspath, dirname

    try:
        location = dirname(
            check_output(['which', 'clp']).strip()).decode('utf-8')
        CoinDir = abspath(join(location, ".."))
    except:
        pass
            
def get_libs():
    '''
    Return a list of distinct library names used by ``dependencies``.
    '''
    libs = []

    try:
        from subprocess import check_output
        
        flags = (check_output(['pkg-config', '--libs', 'dip'])
                 .strip().decode('utf-8'))
        libs = [flag[2:] for flag in flags.split()
                if flag.startswith('-l')]
        libDirs = [flag[2:] for flag in flags.split()
                   if flag.startswith('-L')]
        flags = (check_output(['pkg-config', '--cflags', 'dip'])
                 .strip().decode('utf-8'))
        incDirs = [flag[2:] for flag in flags.split() if
                   flag.startswith('-I')]

    except:
        if CoinDir != None:
            
            if operatingSystem == 'windows':
                if os.path.exists(join(CoinDir, 'lib', 'Cbc.lib')):
                    libs = ['Decomp', 'OsiSym', 'Sym', 'OsiCbc', 'CbcSolver', 'Cbc', 'Cgl',
                            'OsiClp', 'ClpSolver',  'Clp', 'Osi', 'Alps', 'CoinUtils']
                else:
                    libs = ['libDecomp', 'libOsiSym', 'libSym', 'libOsiCbc', 'libCbcSolver',
                            'libCbc', 'libCgl', 'libOsiClp', 'libClpSolver',  'libClp', 'libOsi',
                            'libAlps', 'libCoinUtils']
            else:
                libs = ['Decomp', 'OsiSym', 'Sym', 'OsiCbc', 'CbcSolver', 'Cbc', 'Cgl',
                        'OsiClp', 'ClpSolver',  'Clp', 'Osi', 'Alps', 'CoinUtils']
                
            libDirs = [join(CoinDir, 'lib')]
            incDirs = [join(CoinDir, 'include', 'coin')] 
                
        else:
            raise Exception('''
            Could not find location of COIN installation.
            Please ensure that either 
            * COIN_INSTALL_DIR is set to the location of the installation,
            * PKG_CONFIG_PATH points to the location of the .pc files, or
            * The cbc executable is in your executable path and is installed
            at the same location as the libraries. 
            ''')

    return libs, libDirs, incDirs

operatingSystem = sys.platform
if 'linux' in operatingSystem:
    operatingSystem = 'linux'
elif 'darwin' in operatingSystem:
    operatingSystem = 'mac'
elif 'win' in operatingSystem:
    operatingSystem = 'windows'

libs, libDirs, incDirs = get_libs()

macros = [('__DECOMP_LP_CLP__', None)]

files = ['DippyDecompAlgo.cpp',
         'DippyDecompApp.cpp',
         'DippySolve.cpp',
         'DippyModule.cpp',
         'DippyPythonUtils.cpp',
         ]

sources = [join('src', 'dippy', f) for f in files]

modules=[Extension('coinor.dippy._dippy', 
                   sources, 
                   libraries=libs,
                   include_dirs=incDirs,
                   library_dirs=libDirs,
                   define_macros=macros)]

cvpmp_instance_files = [join('Instances', f) for f in
                        listdir('src/dippy/examples/cvpmp/Instances')]

setup(name=PROJECT,
      version=VERSION,
      description=DESC,
      long_description=read_file('README.md'),
      long_description_content_type='text/markdown',
      author='''Michael O'Sullivan, Qi-Shan Lim, Stuart Mitchell''',
      author_email='michael.osullivan@auckland.ac.nz',
      maintainer='Ted Ralphs',
      maintainer_email='ted@lehigh.edu',
      url=URL,
      license='Eclipse Public License',
      namespace_packages=['coinor'],
      #There must be a better way
      packages=[pkg.replace('src','coinor') for pkg in find_packages()],
      package_dir = {'coinor': 'src'},
      package_data = {'coinor.dippy.examples.cvpmp': cvpmp_instance_files},
      install_requires=['pulp>=1.5.4','coinor.gimpy>=2.0.0'],
      ext_modules=modules
     )





from __future__ import absolute_import
__title__   = 'Main module of B&P-cut for the CVPMP and CVPCP'
__version__ = '1.0 Nov 2013'
__author__  = 'Dago Quevedo'
__email__   = 'dago@yalma.fime.uanl.mx'

from . import  draw
from . import  Global

from .CVPMP import *
from .readInstance import *

def main():

    args = parseArgs()
    
    id,n,p,d,s,w,cxy,V,mytype,id = read(args.instance)
    
    Init(n,p,d,s,w,V)
    z, solution = Solver(args)

    #display solution
    data = 'out.dat'
    f = open(data,'w')
        
    for s in solution:
        i=s[0]
        for j in s[1]:
            if i == j:
                mytype = 1
            else:
                mytype = 0

            f.write('%d\t%d\t%d\t%d\t%d\n'%
                        (mytype, j, i, cxy[j][0], cxy[j][1]))

    f.close();
    draw.draw(data, mytype, id)

main()



from __future__ import division
from __future__ import absolute_import
from builtins import range
from past.utils import old_div
import unittest
from pulp import *
import coinor.dippy as dippy

from .dippy_tests import DippyTestCase


class TestFacilityProblem(DippyTestCase):

    def setUp(self):
        """
        sets up the coke problem as a unittest
        """
        self.prob, self.branch, self.cuts = create_facility_problem()
        self.tol = pow(pow(2, -24), old_div(2.0, 3.0))

    def test_pulp_solve(self):
        """
        tests that pulp can solve the problem
        """
        status = self.prob.solve()
        self.assertEqual(status, LpStatusOptimal)
        self.assertAlmostEqual(self.prob.objective.value(), 212.0)
        self.variable_feasibility_test(self.prob)
        self.constraint_feasibility_test(self.prob)

    #~ def test_dippy_branch(self):
        #~ """
        #~ tests that the custom branch can solve the problem
        #~ """
        #~ self.prob.branch_method = self.branch
        #~ dippy.Solve(self.prob, {
            #~ 'TolZero': '%s' % self.tol,
            #~ })
        #~ self.assertAlmostEqual(self.prob.objective.value(), 212.0)
        #~ self.variable_feasibility_test(self.prob)
        #~ self.constraint_feasibility_test(self.prob)

    def test_dippy_cuts(self):
        """
        tests that the custom branch can solve the problem
        """
        self.prob.generate_cuts = self.cuts
        dippy.Solve(self.prob, {
            'TolZero': '%s' % self.tol,
            })
        self.assertAlmostEqual(self.prob.objective.value(), 212.0)
        self.variable_feasibility_test(self.prob)
        self.constraint_feasibility_test(self.prob)

    def test_dippy_branch_cuts(self):
        """
        tests that the custom branch can solve the problem
        """
        self.prob.branch_method = self.branch
        self.prob.generate_cuts = self.cuts
        dippy.Solve(self.prob, {
            'TolZero': '%s' % self.tol,
            })
        self.assertAlmostEqual(self.prob.objective.value(), 212.0)
        self.variable_feasibility_test(self.prob)
        self.constraint_feasibility_test(self.prob)



def create_facility_problem():
    """
    creates and returns the facility problem
    """

    from math import floor, ceil

    tol = pow(pow(2, -24), old_div(2.0, 3.0))

    # The requirements for the products

    REQUIREMENT = {
        1 : 66,
        2 : 4,
        3 : 85,
        4 : 93,
        5 : 68,
        6 : 76,
        7 : 74,
        8 : 39,
        9 : 66,
        10 : 17,
    }
    # Set of all products
    PRODUCTS = list(REQUIREMENT.keys())
    PRODUCTS.sort()

    # Set of all locations

    LOCATIONS = [ 1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
    LOCATIONS.sort()

    # The capacity of the facilities

    CAPACITY = 100

    prob = dippy.DipProblem("Facility_Location")

    assign_vars = LpVariable.dicts("AtLocation",
                  [(i, j) for i in LOCATIONS
                          for j in PRODUCTS],
                  0, 1, LpBinary)
    use_vars    = LpVariable.dicts("UseLocation",
                  LOCATIONS, 0, 1, LpBinary)
    waste_vars  = LpVariable.dicts("Waste",
                  LOCATIONS, 0, CAPACITY)

    # objective: minimise waste
    prob += lpSum(waste_vars[i] for i in LOCATIONS), "min"

    # assignment constraints
    for j in PRODUCTS:
        prob += lpSum(assign_vars[(i, j)] for i in LOCATIONS) == 1

    # aggregate CAPACITY constraints
    for i in LOCATIONS:
        prob += lpSum(assign_vars[(i, j)] * REQUIREMENT[j]
                      for j in PRODUCTS) + waste_vars[i] == \
                CAPACITY * use_vars[i]

    # disaggregate CAPACITY constraints
    for i in LOCATIONS:
        for j in PRODUCTS:
            prob += assign_vars[(i, j)] <= use_vars[i]

    # Ordering constraints
    for index, location in enumerate(LOCATIONS):
        if index > 0:
            prob += use_vars[LOCATIONS[index-1]] >= use_vars[location]

    # Anti-symmetry branches
    def choose_antisymmetry_branch(prob, sol):
        num_locations = sum(sol[use_vars[i]] for i in LOCATIONS)
        up   = ceil(num_locations)  # Round up to next nearest integer
        down = floor(num_locations) # Round down
        if  (up - num_locations   > tol) \
        and (num_locations - down > tol): # Is fractional?
            # Down branch: provide upper bounds, lower bounds are default
            down_branch_ub = dict([(use_vars[LOCATIONS[n]], 0)
                                   for n in range(int(down), len(LOCATIONS))])
            # Up branch: provide lower bounds, upper bounds are default
            up_branch_lb = dict([(use_vars[LOCATIONS[n]], 1)
                                 for n in range(0, int(up))])
            # Return the advanced branch to DIP
            return {}, down_branch_ub, up_branch_lb, {}

    def generate_weight_cuts(prob, sol):
    ##    print "In generate_weight_cuts, sol = ", sol

        # Define mu and T for each knapsack
        mu = {}
        S = {}
        for i in LOCATIONS:
            mu[i] = CAPACITY
            S[i] = []

        # Use current assign_var values to assign items to locations
        assigning = True
        while assigning:
            bestValue = 0
            bestAssign = None
            for i in LOCATIONS:
                for j in PRODUCTS:
                    if j not in S[i]: # If this product is not in the subset
                        if (sol[assign_vars[(i, j)]] > bestValue) \
                        and (REQUIREMENT[j] <= mu[i]):
                            # The assignment variable for this product is closer
                            # to 1 than any other product checked, and "fits" in
                            # this location's remaining space
                            bestValue = sol[assign_vars[(i, j)]]
                            bestAssign = (i, j)
            # Make the best assignment found across all products and locactions
            if bestAssign:
                (i,j) = bestAssign
                mu[i] -= REQUIREMENT[j] # Decrease spare CAPACITY at this location
                S[i].append(j) # Assign this product to this location's set
            else:
                assigning = False # Didn't find anything to assign - stop

        # Generate the weight cuts from the sets found above
        new_cuts = []
        for i in LOCATIONS:
            if len(S[i]) > 0: # If an item assigned to this location
                con = LpAffineExpression() # Start a new constraint
                con += sum(REQUIREMENT[j] * assign_vars[(i, j)]
                                for j in S[i])
                con += sum(max(0, REQUIREMENT[j] - mu[i]) *
                                assign_vars[(i, j)] for j in PRODUCTS
                                if j not in S[i])
                new_cuts.append(con <= CAPACITY - mu[i])

        ##    print new_cuts
        # Return the set of cuts we created to DIP
        return new_cuts

        return sols


    return prob, choose_antisymmetry_branch, generate_weight_cuts


if __name__ == '__main__':
    unittest.main()



from __future__ import absolute_import
from builtins import range
import unittest
from pulp import *
import coinor.dippy as dippy

from .dippy_tests import DippyTestCase


class TestTSPProblem(DippyTestCase):

    def setUp(self):
        self.prob, self.generate_cuts, self.is_solution_feasible = \
                    create_tsp_problem()
        self.tol = 1e-6


    def test_pulp_solve(self):
        """
        tests that pulp can solve the problem
        """
        status = self.prob.solve()
        self.assertEqual(status, LpStatusOptimal)
        self.assertAlmostEqual(self.prob.objective.value(), 14.24, 2)
        self.variable_feasibility_test(self.prob)
        self.constraint_feasibility_test(self.prob)
        # would like this in here but not sure how to do it
        #~ self.assertFalse(self.is_solution_feasible())

    def test_dippy_solve(self):
        """
        tests that dippy can solve the problem
        """
        self.prob.generate_cuts = self.generate_cuts
        self.prob.is_solution_feasible = self.is_solution_feasible
        sol, duals = dippy.Solve(self.prob, {})
        self.assertAlmostEqual(self.prob.objective.value(), 18.39, 2)
        self.variable_feasibility_test(self.prob)
        self.constraint_feasibility_test(self.prob)
        # would like this in here but not sure how to do it
        self.assertTrue(self.is_solution_feasible(self.prob, sol, self.tol))

def create_tsp_problem(doRelaxed=False):
    """
    creates and returns the tsp problem
    """

    from math import sqrt

    # 2d Euclidean TSP with extremely simple cut generation

    # x,y coords of cities
    CITY_LOCS = [(0, 2), (0, 4), (1, 2), (1, 4), \
                 (4, 1), (4, 4), (4, 5),  (5, 0), \
                 (5, 2), (5, 5)]
    CITIES = list(range(len(CITY_LOCS)))

    ARCS = [] # list of arcs (no duplicates)
    ARC_COSTS = {} # distance

    # for each city, list of arcs into/out of
    CITY_ARCS = [[] for i in CITIES]

    # use 2d euclidean distance
    def dist(x1, y1, x2, y2):
        return sqrt((x1-x2)**2 + (y1-y2)**2)

    # construct list of arcs
    for i in CITIES:
        i_x, i_y = CITY_LOCS[i]
        for j in CITIES[i+1:]:
            j_x, j_y = CITY_LOCS[j]
            ARC_COSTS[(i,j)] = dist(i_x, i_y, j_x, j_y)
            ARCS.append((i, j))
            CITY_ARCS[i].append((i, j))
            CITY_ARCS[j].append((i, j))

    prob = dippy.DipProblem()

    arc_vars = LpVariable.dicts("UseArc", ARCS, 0, 1, LpBinary)

    # objective
    prob += sum(ARC_COSTS[x] * arc_vars[x] for x in ARCS)

    # degree constraints
    for city in CITIES:
        prob += sum(arc_vars[x] for x in CITY_ARCS[city]) == 2

    # generate subtour elimination constraints

    # dictionary for symmetric arcs, can be
    # accessed using (i, j) and (j, i)
    symmetric = {}
    for i in CITIES:
        for j in CITIES:
            if i < j:
                symmetric[(i, j)] = (i, j)
                symmetric[(j, i)] = (i, j)

    def generate_cuts(prob, sol):
        cons = []
        not_connected = set(CITIES)

        while not_connected:
     #       print "not_connected", [n for n in not_connected]
            start = not_connected.pop()
            nodes, arcs = get_subtour(sol, start)
            if len(nodes) == len(arcs) and \
               len(nodes) < len(CITIES):
                cons.append( sum(arc_vars[a] for a in arcs) \
                             <= len(arcs) - 1 )
     #       print "nodes", [n for n in nodes]
     #       print "arcs", [a for a in arcs]
            nodes.remove(start)
            not_connected -= nodes

    ##    print "# cons = ", len(cons)
    ##    print "cons = ", [ con for con in cons ]
        return cons

    def is_solution_feasible(prob, sol, tol):
        nodes, arcs = get_subtour(sol, 0)
    #    print "Checking: # nodes = ", len(nodes), \
    #          ", # cities = ", len(CITIES)
    #    print "nodes", [n for n in nodes]

        return len(nodes) == len(arcs) and \
               len(nodes) == len(CITIES)

    def get_subtour(sol, node):
        # returns: list of nodes and arcs
        # in subtour containing node
        nodes = set([node])
        arcs = set([])
        not_processed = set(CITIES)
        to_process = set([node])

        tol = 1e-6
        one = 1 - tol

        while to_process:
            c = to_process.pop()
            not_processed.remove(c)
            new_arcs = [ symmetric[(c, i)] \
                         for i in not_processed \
                         if sol[ \
                             arc_vars[symmetric[(c, i)]]]
                         > one]
            new_nodes = [ i for i in not_processed \
                          if symmetric[(i, c)] in new_arcs ]
     #       print "new_nodes", [n for n in new_nodes]
     #       print "new_arcs", [a for a in new_arcs]
            arcs |= set(new_arcs)
            nodes |= set(new_nodes)
            to_process |= set(new_nodes)
     #       print "not_processed", [n for n in not_processed]
     #       print "nodes", [n for n in nodes]
     #       print "arcs", [a for a in arcs]

        return nodes, arcs

    return prob, generate_cuts, is_solution_feasible


if __name__ == '__main__':
    unittest.main()



from __future__ import print_function
from builtins import str
from builtins import range
from builtins import object
import argparse
from pulp import LpVariable, lpSum, LpBinary, LpStatusOptimal

try:
    from src.dippy import DipProblem, Solve
    from src.dippy.examples.gen_func import *
except ImportError:
    from coinor.dippy import DipProblem, Solve
    from coinor.dippy.examples.gen_func import *

class CokeProb(object):
    def __init__(self, supply, demand, LOCATIONS, build_costs,
                 conversion_factor, transport_costs):
        self.MINES     = list(supply.keys())
        self.MINES.sort()
        self.CUSTOMERS = list(demand.keys())
        self.CUSTOMERS.sort()
        self.LOCATIONS = LOCATIONS
        self.SIZES     = list(build_costs.keys())
        self.SIZES.sort()
        self.ARCS      = list(transport_costs.keys())
        self.conversion_factor = conversion_factor
        self.supply            = supply
        self.demand            = demand
        self.build_costs       = build_costs
        self.transport_costs   = transport_costs

def parseArgs():
    
    parser = argparse.ArgumentParser(
        description='Solve a steel manufacturing problem.')
    parser.add_argument('--module', '-m', metavar = 'module name', 
                        help='name of the Python module from which to import data',
                        default = 'coinor.dippy.examples.coke.coke_data')
    parser.add_argument('--advancedBranch', action='store_true', 
                        help='enable advanced branching')

    addDippyArgs(parser)

    args = parser.parse_args()

    return args

def formulate(cp):

    prob = DipProblem("Coke", display_mode = 'xdot', display_interval = None)

    # create variables
    LOC_SIZES = [(l, s) for l in cp.LOCATIONS
                        for s in cp.SIZES]
    buildVars = LpVariable.dicts("Build", LOC_SIZES, cat=LpBinary)

    # create arcs
    flowVars = LpVariable.dicts("Arcs", cp.ARCS)
    BIG_M = max(sum(cp.supply.values()), sum(cp.demand.values()))
    for a in cp.ARCS:
        flowVars[a].bounds(0, BIG_M)

    # objective
    prob += 1e6 * lpSum(buildVars[(l, s)] * cp.build_costs[s] \
                        for (l, s) in LOC_SIZES) + \
                  lpSum(flowVars[(s, d)] * cp.transport_costs[(s, d)] \
                        for (s, d) in cp.ARCS), "min"

    # plant availability - assumes that SIZES are numeric,
    # which they should be
    for loc in cp.LOCATIONS:
        prob += lpSum(flowVars[(loc, i)] for i in cp.CUSTOMERS) \
             <= lpSum(buildVars[(loc, s)] * s for s in cp.SIZES)

    # one size
    for loc in cp.LOCATIONS:
        prob += lpSum(buildVars[(loc, s)] for s in cp.SIZES) == 1

    # conserve flow (mines)
    # flows are in terms of tonnes of coke
    for m in cp.MINES:
        prob += lpSum(flowVars[(m, j)] for j in cp.LOCATIONS) \
             <= cp.supply[m]

    # conserve flow (locations)
    # convert from coal to coke
    for loc in cp.LOCATIONS:
        prob += lpSum(flowVars[(m, loc)] for m in cp.MINES) - \
                cp.conversion_factor * \
                lpSum(flowVars[(loc, c)] for c in cp.CUSTOMERS) \
             >= 0

    for c in cp.CUSTOMERS:
        prob += lpSum(flowVars[(loc, c)] for loc in cp.LOCATIONS) \
             >= cp.demand[c]

    prob.cp = cp
    prob.buildVars = buildVars
    prob.flowVars = flowVars
    
    return prob
  
def do_branch(prob, sol):
    tol       = prob.tol
    LOCATIONS = prob.cp.LOCATIONS
    SIZES     = prob.cp.SIZES
    buildVars = prob.buildVars
    
    for loc in LOCATIONS:
        sol_size = sum(sol[buildVars[loc, size]] * \
                       size for size in SIZES)
        # smallest index and size larger than sol_size
        if abs(sol_size - SIZES[-1]) < tol:
            continue
        i, bigger = [(i, s) for i, s in enumerate(SIZES) \
                     if s > sol_size][0]
        if i == 0:
            smaller = 0
        else:
            smaller = SIZES[i-1]

        down_lbs = {}
        down_ubs = {}
        up_lbs = {}
        up_ubs = {}
        if bigger - sol_size > tol and sol_size - smaller > tol:
            down_ubs = dict([(buildVars[loc, SIZES[j]], 0) 
                             for j in range(i, len(SIZES))])
            up_ubs = dict([(buildVars[loc, SIZES[j]], 0) 
                           for j in range(0, i)])

            return down_lbs, down_ubs, up_lbs, up_ubs
        
def solve(prob, args):

    if args.advancedBranch:
        prob.branch_method = do_branch

    if args.algo != 'Cut':
        print("This application only works with the cutting plane method")
        
    dippyOpts = addDippyOpts(args)

    status, message, primals, duals = Solve(prob, dippyOpts)
    
    if status == LpStatusOptimal:
        return dict((var, var.value()) for var in prob.variables())
    else:
        return None

def read_table(data, coerce, transpose=False):
    lines = data.splitlines()
    headings = lines[1].split()
    result = {}
    for row in lines[2:]:
        items = row.split()
        for i, item in enumerate(items[1:]):
            if transpose: key = (headings[i], items[0])
            else: key = (items[0], headings[i])
            result[key] = coerce(item)
    return result

def print_table(rows, cols, fn):
    print("\t", "\t".join(cols))
    for r in rows:
        print(r,"\t", "\t".join(str(fn(r,c)) for c in cols))

def print_var_table(rows, cols, var, fn=lambda x: x):
    print_table(rows, cols, lambda x, y:
                fn(var[(x,y)].varValue))
